/*
 *  Copyright (C) 2009-2021 Christophe DEVALLAND
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef COM_SUN_STAR_SHEET_ADDIN_CMATHOOOCAS_HXX
#define COM_SUN_STAR_SHEET_ADDIN_CMATHOOOCAS_HXX

#include <com/sun/star/uno/XComponentContext.hpp>
#include <com/sun/star/sheet/addin/XCmathOOoCASAddin.hpp>
#include <com/sun/star/sheet/XAddIn.hpp>
//#include <com/sun/star/lang/XInitialization.hpp>
#include <com/sun/star/lang/XServiceInfo.hpp>
#include <com/sun/star/lang/XServiceName.hpp>
#include <com/sun/star/lang/XLocalizable.hpp>
//#include <cppuhelper/factory.hxx>
//#include <cppuhelper/implementationentry.hxx>
//#include <cppuhelper/supportsservice.hxx>
//#include <com/sun/star/lang/IllegalArgumentException.hpp>
#include <com/sun/star/lang/Locale.hpp>
#include <cppuhelper/implbase4.hxx>
#include <giac/config.h>
#include <giac/giac.h>				// pour le calcul formel
#include <giac/input_lexer.h>		// pour ajouter les traductions (fichier keywords)

#define IMPLEMENTATION_NAME "com.sun.star.sheet.addin.CmathOOoCASAddinImpl"
#define NB_FUNCTIONS 90

using namespace ::rtl; 					// pour OUString
using namespace ::com::sun::star::uno; 	// pour basic types

class CmathOOoCASAddinImpl : public cppu::WeakImplHelper4
<
    com::sun::star::sheet::addin::XCmathOOoCASAddin,
    com::sun::star::sheet::XAddIn,
    com::sun::star::lang::XServiceName,
    com::sun::star::lang::XServiceInfo
>
{

private:
	static const sal_Int32 nNumFunctions = NB_FUNCTIONS;
    com::sun::star::lang::Locale aFuncLoc;
    
public:

    CmathOOoCASAddinImpl()
    {
        printf("DEBUG>>> Created an instance of CmathOOoCASAddinImpl %p\n", this); fflush(stdout);
    }

    // CmathOOoCasAddin --------------------------------------------------------------------
    // Les fonctions de CmathOOoCAS agissant sur une cellule
    virtual Any SAL_CALL cas( Any const & contenu );
    virtual Any SAL_CALL simplifier( Any const & contenu ) ;
    virtual Any SAL_CALL cexact( Any const & contenu ) ;
    virtual Any SAL_CALL csqrt( Any const & contenu ) ;
    virtual Any SAL_CALL csin( Any const & contenu ) ;
    virtual Any SAL_CALL ccos( Any const & contenu ) ;
    virtual Any SAL_CALL ctan( Any const & contenu ) ;
    virtual Any SAL_CALL cln( Any const & contenu ) ;
    virtual Any SAL_CALL cexp( Any const & contenu ) ;
    virtual Any SAL_CALL cabs( Any const & contenu ) ;
    virtual Any SAL_CALL conj( Any const & contenu ) ;
    virtual Any SAL_CALL arg( Any const & contenu ) ;
    virtual Any SAL_CALL re( Any const & contenu ) ;
    virtual Any SAL_CALL im( Any const & contenu ) ;
    virtual Any SAL_CALL evalc( Any const & contenu ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL idivis(Any const & contenu ) ;
    virtual Any SAL_CALL factorial( Any const & contenu ) ;
    virtual Any SAL_CALL factoriser( Any const & contenu ) ;
    virtual Any SAL_CALL factoriserSurC( Any const & contenu ) ;
    virtual Any SAL_CALL developper( Any const & contenu ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL divis(Any const & contenu ) ;
    virtual Any SAL_CALL factoriserEntier( Any const & contenu ) ;
    virtual Any SAL_CALL isprime( Any const & contenu ) ;
    virtual Any SAL_CALL carcsin( Any const & contenu ) ;
    virtual Any SAL_CALL carccos( Any const & contenu ) ;
    virtual Any SAL_CALL carctan( Any const & contenu ) ;
    virtual Any SAL_CALL prevprime( Any const & contenu ) ;
    virtual Any SAL_CALL nextprime( Any const & contenu ) ;
    virtual Any SAL_CALL isPseudoPrime( Any const & contenu ) ;
    virtual Any SAL_CALL euler( Any const & contenu ) ;
    virtual Any SAL_CALL latex( Any const & contenu ) ;
    virtual Any SAL_CALL mathml( Any const & contenu ) ;
    virtual Any SAL_CALL developperTranscendant( Any const & contenu ) ;
    virtual Any SAL_CALL ccosh( Any const & contenu ) ;
    virtual Any SAL_CALL csinh( Any const & contenu ) ;
    virtual Any SAL_CALL ctanh( Any const & contenu ) ;
    virtual Any SAL_CALL carccosh( Any const & contenu ) ;
    virtual Any SAL_CALL carcsinh( Any const & contenu ) ;
    virtual Any SAL_CALL carctanh( Any const & contenu ) ;
    virtual Any SAL_CALL ccotan( Any const & contenu ) ;
    virtual Any SAL_CALL carccotan( Any const & contenu ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL casCompil(Any const & contenu ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL casFonction(OUString const & nomFonction, const Sequence<  Any  > &aValList ) ;

    // Les fonctions de CmathOOoCAS agissant sur plusieurs cellules
    virtual Any SAL_CALL csub( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Any SAL_CALL cpow( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Any SAL_CALL cdiv( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Any SAL_CALL evalf( Any const & contenu1, Any const & contenu2 ) ;
    virtual Any SAL_CALL subst( Any const & contenu1 , Any const & contenu2 , Any const & contenu3) ;
    virtual Any SAL_CALL deriver( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Any SAL_CALL integrer( Any const & contenu1 , Any const & contenu2 , Any const & contenu3, Any const & contenu4) ;
    virtual Any SAL_CALL csomme( Any const & contenu1 , Any const & contenu2 , Any const & contenu3, Any const & contenu4) ;
    virtual Any SAL_CALL cproduit( Any const & contenu1 , Any const & contenu2 , Any const & contenu3, Any const & contenu4) ;
    virtual Any SAL_CALL limite( Any const & contenu1 , Any const & contenu2 , Any const & contenu3, Any const & contenu4) ;
    virtual Sequence< Sequence< Any > > SAL_CALL czeros( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL zeros( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL makelist( Any const & contenu1 , Any const & contenu2 , Any const & contenu3, Any const & contenu4, Any const & contenu5) ;
    virtual Any SAL_CALL iquo( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Any SAL_CALL irem( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Any SAL_CALL quo( Any const & contenu1 , Any const & contenu2 , Any const & contenu3) ;
    virtual Any SAL_CALL rem( Any const & contenu1 , Any const & contenu2 , Any const & contenu3) ;
    virtual Sequence< Sequence< Any > > SAL_CALL resoudre( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL resoudreDansC( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL resoudreNumerique( Any const & contenu1 , Any const & contenu2 , Any const & contenu3, Any const & contenu4) ;
    virtual Any SAL_CALL gcd( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Any SAL_CALL lcm( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Any SAL_CALL formeCanonique( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Any SAL_CALL taylor( Any const & contenu1 , Any const & contenu2 , Any const & contenu3) ;
    virtual Sequence< Sequence< Any > > SAL_CALL iegcd( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL iabcuv( Any const & contenu1 , Any const & contenu2 , Any const & contenu3) ;
    virtual Sequence< Sequence< Any > > SAL_CALL egcd( Any const & contenu1 , Any const & contenu2 , Any const & contenu3) ;
    virtual Sequence< Sequence< Any > > SAL_CALL abcuv( Any const & contenu1 , Any const & contenu2 , Any const & contenu3, Any const & contenu4) ;
    virtual Any SAL_CALL fmax( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Any SAL_CALL fmin( Any const & contenu1 , Any const & contenu2 ) ;
    virtual Any SAL_CALL dsolve( Any const & contenu1 , Any const & contenu2 , Any const & contenu3, Any const & contenu4) ;

    // Les fonctions de CmathOOoCAS agissant sur une zone de cellules.
    virtual Any SAL_CALL cadd(const Sequence<  Any  > &aValList ) ;
    virtual Any SAL_CALL cmul(const Sequence<  Any  > &aValList ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL linsolve(const Sequence< Sequence< Any > > &aArrayCell ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL egv(const Sequence< Sequence< Any > > &aArrayCell ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL eigenvals(const Sequence< Sequence< Any > > &aArrayCell ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL matinv(const Sequence< Sequence< Any > > &aArrayCell ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL matadd(const Sequence< Sequence< Any > > &aArrayCell1,const Sequence< Sequence< Any > > &aArrayCell2 ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL matsub(const Sequence< Sequence< Any > > &aArrayCell1, const Sequence< Sequence< Any > > &aArrayCell2 ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL matmult(const Sequence< Sequence< Any > > &aArrayCell1, const Sequence< Sequence< Any > > &aArrayCell2 ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL matpow(const Sequence< Sequence< Any > > &aArrayCell, Any const & contenu ) ;
    virtual Any SAL_CALL det(const Sequence< Sequence< Any > > &aArrayCell ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL cas1(const Sequence<  Any  > &aValList ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL cas2(const Sequence<  Any  > &aValList ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL cas3(const Sequence<  Any  > &aValList ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL cas4(const Sequence<  Any  > &aValList ) ;
    virtual Sequence< Sequence< Any > > SAL_CALL cas5(const Sequence<  Any  > &aValList ) ;

    // fonction de lecture des synonymes des commandes giac
    virtual void SAL_CALL cmath_update_lexer_localization(const std::vector<int> & v,std::map<std::string,std::string> &lexer_map,std::multimap<std::string,giac::localized_string> &back_lexer_map, std::string &synonymsFile) ;

    // gestion des erreurs de syntaxe
    virtual Any SAL_CALL syntaxErreur(void);
    virtual Sequence< Sequence< Any > > SAL_CALL syntaxErreurMatrix(void);
    // gestion des fonctions non encore implémentées
    virtual Any SAL_CALL notYetImplemented(void);
    virtual Sequence< Sequence< Any > > SAL_CALL notYetImplementedMatrix(void);

    // XAddIn
    OUString SAL_CALL getProgrammaticFuntionName( const OUString& aDisplayName );
    OUString SAL_CALL getDisplayFunctionName( const OUString& aProgrammaticName );
    OUString SAL_CALL getFunctionDescription( const OUString& aProgrammaticName );
    OUString SAL_CALL getDisplayArgumentName( const OUString& aProgrammaticFunctionName, sal_Int32 nArgument );
    OUString SAL_CALL getArgumentDescription( const OUString& aProgrammaticFunctionName, sal_Int32 nArgument );
    OUString SAL_CALL getProgrammaticCategoryName( const OUString& aProgrammaticFunctionName );
    OUString SAL_CALL getDisplayCategoryName( const OUString& aProgrammaticFunctionName );
    virtual void SAL_CALL setLocale( const com::sun::star::lang::Locale& aLocale ) { aFuncLoc = aLocale; }
    virtual com::sun::star::lang::Locale SAL_CALL getLocale() { return aFuncLoc; }

    // XServiceInfo methods
    OUString SAL_CALL getImplementationName() ;
    sal_Bool SAL_CALL supportsService( const OUString& aServiceName ) ;
    Sequence< OUString > SAL_CALL getSupportedServiceNames() ;

    // XServiceName
    OUString SAL_CALL getServiceName();
};

Sequence< OUString > SAL_CALL getSupportedServiceNames_CmathOOoCASAddinImpl();
Reference< XInterface > SAL_CALL create_CmathOOoCASAddinImpl( const Reference< XComponentContext > & rContext);    

#endif
