/**C++
*/
/*
 *  Copyright (C) 2009-2021 Christophe DEVALLAND
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "CmathOOoCAS.hxx"
#include <cppuhelper/supportsservice.hxx>
#include <rtl/ustring.hxx>
#include <rtl/math.hxx>
#include <fstream>


using namespace ::rtl; 					// pour OUString
using namespace ::com::sun::star; 		// pour odk interfaces
using namespace ::com::sun::star::uno; 	// pour basic types
using namespace ::com::sun::star::sheet;
using namespace std;
using namespace giac;  					// pour le calcul formel

#define BASE_SERVICE_NAME "com.sun.star.sheet.AddIn"
#define SERVICE_NAME "com.sun.star.sheet.addin.CmathOOoCASAddin"
 
// variable globale de context
giac::context global_context;
giac::context * contextptr = & global_context;

// variable globale comptant le nombre d'aides disponibles dans la ligne de commande CAS
sal_Int32 nbAides=0;
sal_Int32 nbSynonymes=0;

// XServiceName
OUString SAL_CALL CmathOOoCASAddinImpl::getServiceName()
{
    // name of specific AddIn service
    return SERVICE_NAME;
}

Sequence< OUString > SAL_CALL getSupportedServiceNames_CmathOOoCASAddinImpl()
{
    Sequence< OUString > names(2);
    names[0] = OUString ( BASE_SERVICE_NAME );
    names[1] = OUString ( SERVICE_NAME );
    return names;
}

Reference< XInterface > SAL_CALL create_CmathOOoCASAddinImpl( Reference< XComponentContext > const & xContext )
{
    //return static_cast< ::cppu::OWeakObject * >( new CmathOOoCASAddinImpl( xContext ) );
    return (cppu::OWeakObject*) new CmathOOoCASAddinImpl();
}


// Implementation of the recommended/mandatory interfaces of a UNO component.
// XServiceInfo
OUString SAL_CALL CmathOOoCASAddinImpl::getImplementationName() 
{
    // unique implementation name
    return OUString(IMPLEMENTATION_NAME);
}


sal_Bool SAL_CALL CmathOOoCASAddinImpl::supportsService( const OUString& rServiceName )
    
{
	printf("DEBUG>>> supportService(%s)\n", OUStringToOString( rServiceName, RTL_TEXTENCODING_ASCII_US ).getStr()); fflush(stdout);
    return ( rServiceName == BASE_SERVICE_NAME || rServiceName == SERVICE_NAME );
}

Sequence< OUString > CmathOOoCASAddinImpl::getSupportedServiceNames()  
{
	printf("DEBUG>>> inside getSupportedServiceNames()\n"); fflush(stdout);
	return getSupportedServiceNames_CmathOOoCASAddinImpl();
}

const OUString type_double = OUString::createFromAscii("double");
const OUString type_long = OUString::createFromAscii("long");
const OUString type_short = OUString::createFromAscii("short");
const OUString type_byte = OUString::createFromAscii("byte");
const OUString type_string = OUString::createFromAscii("string");

// Localisation 
// Pour le moment : français,allemand et par défaut anglais.


// les fonctions de CmathOOoCAS --------------------------------------------------------------

bool CelluleVide ( Any const & contenu ) 
{
	bool cell_vide=false;
	OUString type_name=contenu.getValueTypeName();
	if (type_name==type_string)
	{	
		OUString chaine;
		contenu >>= chaine;
		if (chaine.getLength()==0)
			return true;
	}
	return false;
}


giac::gen Cellule2Gen( Any const & contenu )
{
	// convertit le contenu d'une cellule en expression de type giac
	gen e;
	setlocale(LC_NUMERIC, "POSIX");	// pour que le séparateur décimal soit le point pour giac et non la virgule sur un OOo français
	OUString type_name=contenu.getValueTypeName();
	std::cout << "Cellule2Gen : type = "  << type_name << ", contenu = " << contenu << std::endl;	
	if (type_name==type_double)
	{
		double valeur;		
		contenu >>= valeur;
		long partie_entiere = (long)(valeur);
		if (partie_entiere==valeur)
		{
			// retrouver le type entier perdu par OOo/LO
			e=partie_entiere;		
			std::cout << "Conversion en long : " << e << std::endl;
		}
		else
		{
			e=valeur;
			std::cout << "Conversion en double : " << e << std::endl;
		}
	}
	else
	{
		if (type_name==type_long||type_name==type_short||type_name==type_byte)
		{
			// c'est un entier
			long valeur;
			contenu >>= valeur;
			e=valeur;
			std::cout << "Conversion en long : " << e << std::endl;
		}
		else
		{	
			if (type_name==type_string)
			{
				OUString chaine;
				contenu >>= chaine;
				OString aOString = OUStringToOString (chaine,RTL_TEXTENCODING_UTF8);			
				// autre possibilité avec LO : aOString = chaine.toUtf8();
				gen temp(aOString.getStr(),contextptr);
				e=temp;
				std::cout << "Conversion en string : " << e << std::endl;
			}
			else
			{
				// les autres types ne sont pas gérés.
				// encore jamais vu...
				OUString chaine;
				chaine=OUString::createFromAscii("type ")+type_name+OUString::createFromAscii(" non gere.");
				OString aOString = OUStringToOString (chaine,RTL_TEXTENCODING_UTF8);
				gen temp(aOString.getStr(),contextptr);
				e=temp;
			}
		}
	}
	return e;
}

OString Gen2String( giac::gen & e)
{
	static std::string es;
	es=e.print(contextptr);
	return(es.c_str());
}

giac::matrice Cellules2Gen( const Sequence< Sequence< Any > >  & aArraySeq )
{
	// convertit le contenu d'une zone de cellules en matrice de type giac
	// gen e;
	setlocale(LC_NUMERIC, "POSIX");	// pour que le séparateur décimal soit le point pour giac et non la virgule sur un OOo français
	sal_Int32 n1, n2;
	sal_Int32 nE1 = aArraySeq.getLength();
	sal_Int32 nE2 = aArraySeq[0].getLength(); // OOo ne renvoie que des tableaux rectangulaires réguliers
	matrice m; 
	m.reserve(nE1);

	// std::cout << "Cellules2Gen any : "  << std::endl;
	// std::cout << "nblig : " << nE1 << std::endl;
	// std::cout << "nbcol : " << nE2 << std::endl;

	for( n1 = 0 ; n1 < nE1 ; n1++ )
	{
		const Sequence< Any >	rList = aArraySeq[ n1 ];
		nE2 = rList.getLength();
		vecteur l(nE2);
		const Any*	pList = rList.getConstArray();
		for( n2 = 0 ; n2 < nE2 ; n2++ )
		{
			gen temp;
			temp = Cellule2Gen(pList[n2]);
			// std::cout << "lu : " << Gen2String(temp) << std::endl;
			l[n2] = temp;
			// (*m[n1]._VECTptr)[n2]=temp;
		}
		m.push_back(l);
	}
	return m;
}

giac::matrice Cellules2Gen( const Sequence< Sequence< double > >  & aArraySeq )
// seul OOoBasic semble pouvoir retourner ce genre de cellules
{
	// convertit le contenu d'une zone de cellules en matrice de type giac
	// gen e;
	setlocale(LC_NUMERIC, "POSIX");	// pour que le séparateur décimal soit le point pour giac et non la virgule sur un OOo français
	sal_Int32 n1, n2;
	sal_Int32 nE1 = aArraySeq.getLength();
	sal_Int32 nE2 = aArraySeq[0].getLength(); // OOo ne renvoie que des tableaux rectangulaires réguliers
	matrice m; 
	m.reserve(nE1);

	// std::cout << "Cellules2Gen double : "  << std::endl;
	// std::cout << "nblig : " << nE1 << std::endl;
	// std::cout << "nbcol : " << nE2 << std::endl;

	for( n1 = 0 ; n1 < nE1 ; n1++ )
	{
		const Sequence< double > rList = aArraySeq[ n1 ];
		nE2 = rList.getLength();
		vecteur l(nE2);
		const double*	pList = rList.getConstArray();
		for( n2 = 0 ; n2 < nE2 ; n2++ )
		{
			gen temp;
			temp = Cellule2Gen(makeAny(pList[n2]));
			// std::cout << "lu : " << Gen2String(temp) << std::endl;
			l[n2] = temp;
			// (*m[n1]._VECTptr)[n2]=temp;
		}
		m.push_back(l);
	}
	return m;
}

giac::matrice Cellules2Gen( const Sequence< Sequence< OUString > >  & aArraySeq )
// seul OOoBasic semble pouvoir retourner ce genre de cellules
{
	// convertit le contenu d'une zone de cellules en matrice de type giac
	// gen e;
	setlocale(LC_NUMERIC, "POSIX");	// pour que le séparateur décimal soit le point pour giac et non la virgule sur un OOo français
	sal_Int32 n1, n2;
	sal_Int32 nE1 = aArraySeq.getLength();
	sal_Int32 nE2 = aArraySeq[0].getLength(); // OOo ne renvoie que des tableaux rectangulaires réguliers
	matrice m; 
	m.reserve(nE1);

	// std::cout << "Cellules2Gen string : "  << std::endl;
	// std::cout << "nblig : " << nE1 << std::endl;
	// std::cout << "nbcol : " << nE2 << std::endl;

	for( n1 = 0 ; n1 < nE1 ; n1++ )
	{
		const Sequence< OUString >	rList = aArraySeq[ n1 ];
		nE2 = rList.getLength();
		vecteur l(nE2);
		const OUString*	pList = rList.getConstArray();
		for( n2 = 0 ; n2 < nE2 ; n2++ )
		{
			gen temp;
			temp = Cellule2Gen(makeAny(pList[n2]));
			// std::cout << "lu : " << Gen2String(temp) << std::endl;
			l[n2] = temp;
			// (*m[n1]._VECTptr)[n2]=temp;
		}
		m.push_back(l);
	}
	return m;
}

Any Gen2any( giac::gen & e)
{
	std::cout << "Gen2Any : e = "  << e << std::endl;
	
	if (e.type==_INT_)
	{
		std::cout << "Gen2Any : conversion en long = "  << e.val << std::endl;
		return makeAny((sal_Int32)e.val);
	} else
	{
		if (e.type==_DOUBLE_)
		{
			std::cout << "Gen2Any : conversion en double = "  << e._DOUBLE_val << std::endl;
			return makeAny(e._DOUBLE_val);
		} else
		{
			// static std::string es;
			// es=e.print(contextptr);
			// OString oes=es.c_str();
			OUString resultat=OStringToOUString(Gen2String(e),RTL_TEXTENCODING_UTF8);
			//OUString resultat=OUString::createFromAscii(oes);
			return makeAny(resultat);
		}
	} 
}


OString Cellule2String ( Any  contenu )
{
	if (!contenu.hasValue()) return "";
	OUString type_name=contenu.getValueTypeName();
	std::cout << "Cellule2String : type = "  << type_name << ", contenu = " << contenu << std::endl;
	if (type_name==type_double)
	{
		// c'est un double
		double valeur;
		contenu >>= valeur;
		long partie_entiere = (long)(valeur);
		if (partie_entiere==valeur)
		{
			// retrouver le type int perdu par OOo/LO
			std::cout << "Conversion en long : " << OString::number(partie_entiere) << std::endl;
			return OString::number(partie_entiere);
		}
		else
		{
			std::cout << "Conversion en double : " << OString::number(valeur) << std::endl;
			return OString::number(valeur);
		}	
	} 
	else 
	{
		if (type_name==type_long||type_name==type_short||type_name==type_byte)
		{
			// c'est un entier
			long valeur;
			contenu >>= valeur;
			std::cout << "Conversion en long : " << OString::number(valeur) << std::endl;
			return OString::number(valeur);
		}
		else
		{
			// cela peut être : chaine, entier long, complexe, ...
			OUString chaine;
			contenu >>= chaine;
			std::cout << "Conversion en string : " << chaine << std::endl;
			return OUStringToOString (chaine,RTL_TEXTENCODING_UTF8);
		}
	}
}

Sequence< Sequence< Any > > Gen2Matrix( gen e, giac::context * contextptr )
{
	if ( (e.type==_VECT))
	{
		sal_Int32	nE1 = e._VECTptr->size();
		sal_Int32	n1;
		giac::vecteur & v=*e._VECTptr; 
		Sequence< Sequence< Any > > solList2D(nE1);
		for( n1 = 0 ; n1 < nE1 ; n1++ )
		{
			if (v[n1].type==_VECT)
			{
				// matrice
				sal_Int32	nE2;
				sal_Int32	n2;
				nE2 = v[n1]._VECTptr->size();
				giac::vecteur & w=*(v[n1]._VECTptr);
				Sequence< Any >	solList1D(nE2);
				for (n2=0;n2<nE2;n2++){
					solList1D[n2]=Gen2any(w[n2]);
				}
				solList2D[n1]=solList1D;
			}
			else
			{
				// vecteur
				Sequence< Any >	solList1D(1);
				solList1D[0]=Gen2any(v[n1]);
				solList2D[n1]=solList1D;
			} 
		}	
		return solList2D;
	}
	else
	{
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=Gen2any(e);
		solList2D[0]=solList1D;
		return solList2D;
	}
}

Sequence< Sequence< Any > > OUString2AnyMatrix(OUString message)
{	// pas de contenu

	Sequence< Any >	solList1D(1);
	Sequence< Sequence< Any > > solList2D(1);
	solList1D[0]=makeAny(message);
	solList2D[0]=solList1D;
	return solList2D;
}

/*
Sequence< Sequence< Any > > Gen2Vecteur( gen e, giac::context * contextptr )
// renvoie une matrice dont à une seule ligne contenant les vecteurs colonnes de e
// seule la première ligne est affichée par OOoBasic

{
	if ( (e.type==_VECT))
	{
		sal_Int32	nE1 = e._VECTptr->size();
		sal_Int32	n1;
		giac::vecteur & v=*e._VECTptr; 
		Sequence< Sequence< Any > > solList2D(1);
		Sequence< Any >	solList1D(nE1);
		for( n1 = 0 ; n1 < nE1 ; n1++ )
		{
				// vecteur
				// nE2 = 1;
				//	std::vector <Any> z(nE2);

				solList1D[n1]=Gen2any(v[n1]);
		}	
		solList2D[0]=solList1D;
		return solList2D;
	}
	else
	{
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=Gen2any(e);
		solList2D[0]=solList1D;
		return solList2D;
	}
} */


Any CmathOOoCASAddinImpl::syntaxErreur(void)
{
	OUString Erreur;
	if (aFuncLoc.Language.equalsAscii("fr"))
	{
		Erreur=OUString::createFromAscii("Erreur de syntaxe");
	}
	else if (aFuncLoc.Language.equalsAscii("de"))
	{
		Erreur=OUString::createFromAscii("Syntaxfehler");
	}
	else 
	{	
		// default is english 
		Erreur=OUString::createFromAscii("Syntax error");
	}
	return makeAny(Erreur);
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::syntaxErreurMatrix(void)
{	// pas de contenu
	OUString Erreur;
	Sequence< Any >	solList1D(1);
	Sequence< Sequence< Any > > solList2D(1);
	if (aFuncLoc.Language.equalsAscii("fr"))
	{
		Erreur=OUString::createFromAscii("Erreur de syntaxe");
	}
	else if (aFuncLoc.Language.equalsAscii("de"))
	{
		Erreur=OUString::createFromAscii("Syntaxfehler");
	}
	else 
	{	
		// default is english 
		Erreur=OUString::createFromAscii("Syntax error");
	}
	solList1D[0]=makeAny(Erreur);
	solList2D[0]=solList1D;
	return solList2D;
}

Any CmathOOoCASAddinImpl::notYetImplemented(void)
{
	OUString Erreur;
	if (aFuncLoc.Language.equalsAscii("fr"))
	{
		Erreur=OUString::createFromAscii("non disponible");
	}
	else if (aFuncLoc.Language.equalsAscii("de"))
	{
                Erreur=OUString::createFromAscii("nicht implementiert");
	}
	else 
	{	
		// default is english 
		Erreur=OUString::createFromAscii("not implemented");
	}
	return makeAny(Erreur);
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::notYetImplementedMatrix(void)
{	// pas de contenu
	OUString Erreur;
	Sequence< Any >	solList1D(1);
	Sequence< Sequence< Any > > solList2D(1);
	if (aFuncLoc.Language.equalsAscii("fr"))
	{
		Erreur=OUString::createFromAscii("non disponible");
	}
	else if (aFuncLoc.Language.equalsAscii("de"))
	{
                Erreur=OUString::createFromAscii("nicht implementiert");
	}
	else 
	{	
		// default is english 
		Erreur=OUString::createFromAscii("not implemented");
	}
	solList1D[0]=makeAny(Erreur);
	solList2D[0]=solList1D;
	return solList2D;
}

Any CmathOOoCASAddinImpl::cas( Any const & contenu )
// fonction qui ne fait qu'évaluer le contenu d'une cellule
// appelée aussi par writer
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);	
	giac::child_id=1; // certaines fonctions giac (comme laplace, cas_setup...) nécessite ce contexte
	ostringstream oss(ostringstream::out);
	oss.str("");
	logptr(&oss,contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		logptr(0,contextptr);
		// english only with Giac
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	logptr(0,contextptr);
	return Gen2any(e);
}

void createfromascii_unexpected() {
  	// throw 0;
	// nothing to do
}


Sequence< Sequence< Any > > CmathOOoCASAddinImpl::casCompil( Any const & contenu )
// fonction créée pour les instructions giac comportant une compilation de fonction
// renvoie un vecteur contenant :
// 1) le résultat
// 2) le flux d'erreurs
// 3) le numéro de la ligne ou a lieu la 1ère erreur
// 4) le mot qui est source de l'erreur
{
	if (!contenu.hasValue()) return syntaxErreurMatrix();
	// giac::context * contextptr=0;
	// ne pas passer par Cellule2Gen car le flux d'erreur n'est pas géré et cela
	// fait planter OOo
	// e=Cellule2Gen(contenu);	
	giac::child_id=1; // certaines fonctions giac (comme laplace, cas_setup...) nécessite ce contexte
	OUString chaine;
	contenu >>= chaine;
	ostringstream oss(ostringstream::out);
	oss.str("");
	logptr(&oss,contextptr);
	OString aOString = OUStringToOString (chaine,RTL_TEXTENCODING_UTF8);
	gen e(aOString.getStr(),contextptr);
	Sequence< Sequence< Any > > sortie;
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		logptr(0,contextptr);
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=makeAny(OUString::createFromAscii(erreur.what()));
		solList2D[0]=solList1D;
		sortie=solList2D;
		return sortie;
	}
	logptr(0,contextptr);
	Sequence< Any >	solList1D(4);
	Sequence< Sequence< Any > > solList2D(1);
	// renvoie le flux d'erreur dans le 1er élément
	solList1D[0]=makeAny(OUString::createFromAscii(oss.str().c_str()));
	// renvoie le résultat évalué dans le second
	solList1D[1]=Gen2any(e);
	// renvoie le n° de ligne de l'erreur
	sal_Int32 errorLine(giac::first_error_line(contextptr));
	solList1D[2]=makeAny(OUString::number(errorLine));
	// et le mot à l'origine de l'erreur
	const char * tokenName(giac::error_token_name(contextptr).c_str());
        solList1D[3]=makeAny(OStringToOUString(tokenName,RTL_TEXTENCODING_UTF8));
	// solList1D[3]=makeAny(OUString::createFromAscii(tokenName));

	solList2D[0]=solList1D;
	sortie=solList2D;
	return sortie;
}


Sequence< Sequence< Any > > CmathOOoCASAddinImpl::casFonction( OUString const & nomFonction, const Sequence<  Any  > &aValList)
// fonction appelée par OOoBasic pour les fonctions définies par l'utilisateur dans l'éditeur de fonctions.
// renvoie une matrice

{
	sal_Int32 nE1 = aValList.getLength();
	sal_Int32 n1;
	giac::child_id=1; // certaines fonctions giac (comme laplace, cas_setup...) nécessite ce contexte
	OString commandeGiac;
	commandeGiac=OUStringToOString (nomFonction,RTL_TEXTENCODING_UTF8)+"(";
	OString variable;
	// std::cout << "casFonction :" << n1 << " arguments"<< std::endl;
	for(n1=0;n1<nE1;n1++) {
		if (!aValList[n1].hasValue()) {
			variable="";
			// std::cout << "argument vide : " << n1 << std::endl;
		}
		else
		{
			gen temp;
			OUString typeName=aValList[n1].getValueTypeName();
			// std::cout << "index : " << typeName.indexOf(OUString::createFromAscii("[][]"),0) << std::endl;
			// if (typeName.indexOf(OUString::createFromAscii("[][]"),0)>=0)
			Sequence< Sequence< Any > > aArraySeq;
			if ( aValList[n1] >>= aArraySeq )
			{
				// std::cout << "Any OK" << std::endl;
				temp = Cellules2Gen(aArraySeq);
			}
			else
			{
				Sequence< Sequence< double > > dArraySeq;
				if ( aValList[n1] >>= dArraySeq )
				{
					// std::cout << "Double OK" << std::endl;
					// std::cout << "(0,0)=" << OString::valueOf(dArraySeq[0][0]) << std::endl;
					temp = Cellules2Gen(dArraySeq);
				}
				else 
				{
					Sequence< Sequence< OUString > > sArraySeq;
					if ( aValList[n1] >>= sArraySeq )
					{
						// std::cout << "String OK" << std::endl;
						// std::cout << "(0,0)=" << OUStringToOString(sArraySeq[0][0],RTL_TEXTENCODING_UTF8) << std::endl;
						temp = Cellules2Gen(sArraySeq);
					}
					else
					{
						// aValList[n1] ne contient qu'une valeur
						// std::cout << "une valeur, ligne : " << n1 << ", contenu Cellule2String : " << Cellule2String(aValList[ n1 ]) << std::endl;
						std::cout << "une valeur, ligne : " << n1 << ", contenu brut : " << aValList[ n1 ] << std::endl;
						// std::cout << "type : " << OUStringToOString (aValList[n1].getValueTypeName(),RTL_TEXTENCODING_UTF8) << std::endl;
						temp = Cellule2Gen(aValList[ n1 ]);
					}
				}

			}
			//std::cout << "argument : " << n1 << std::endl;
			//std::cout << "type : " << OUStringToOString (aValList[n1].getValueTypeName(),RTL_TEXTENCODING_UTF8) << std::endl;
			variable = Gen2String(temp);
		}
		commandeGiac = commandeGiac + variable;
		if (n1<nE1-1) commandeGiac = commandeGiac + ",";
	}
	commandeGiac+=")";
	gen e(commandeGiac.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		Sequence< Sequence< Any > > sortie;
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=makeAny(OUString::createFromAscii(erreur.what()));
		solList2D[0]=solList1D;
		sortie=solList2D;
		return sortie;
	}
	return Gen2Matrix(e,contextptr);
}
 

Any CmathOOoCASAddinImpl::cabs( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	e=Cellule2Gen(contenu);
	try {
		e=giac::abs(e);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}



Any CmathOOoCASAddinImpl::simplifier( Any const & contenu )
{
	std::cout << "Simplifier : contenu cellule = " << contenu << std::endl;
	if (CelluleVide(contenu)==true) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	std::cout << "Simplifier : contenu avant = " << e << std::endl;
	try {
		e=giac::normal(e,contextptr);
		//e=giac::eval(e,contextptr);
		std::cout << "Simplifier : contenu après = " << e << std::endl;
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::evalf( Any const & contenu1, Any const & contenu2 )  
{
	if (!contenu1.hasValue()) return syntaxErreur();
	
	// giac::context * contextptr=0;
	// sal_Int16 save_decimal_digits=decimal_digits(contextptr);
	// set_decimal_digits(precision,contextptr);
	// gen e;
	// e=Cellule2Gen(contenu1);
	OString evalf;
	evalf=OString("evalf(")+Cellule2String(contenu1);
	/*if (e.type==_DOUBLE_) {
		evalf+=OString::valueOf(e._DOUBLE_val);
	} else {
		evalf+=OString(e.print(contextptr).c_str());
	}*/
	if (contenu2.hasValue())
	{
		OUString type_name=contenu2.getValueTypeName();
		if (type_name==type_double)
		{
			// sal_Int16 precision;
			double valeur;
			contenu2 >>= valeur;
			// precision=(sal_Int16)(valeur);
			evalf=evalf+OString(",")+OString::number(rtl::math::round(valeur));
		}
	}
	evalf+=OString(")");
	gen f;
	try {
		gen g(evalf.getStr(),contextptr);
		f=eval(g,contextptr);
		/*f=e.evalf(1,contextptr);
		if (f.type==_REAL || f.type==_CPLX)
	       		f=accurate_evalf(f,digits2bits(precision));
		set_decimal_digits(save_decimal_digits,contextptr);*/
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(f);
	
/*
	if (f.type==_INT_)
	{
		return makeAny((sal_Int32)f.val);
	} else
	{
		if (f.type==_DOUBLE_)
		{
			double arrondi=(double) f._DOUBLE_val;
			// arrondi= rtl::math::round( arrondi, precision, rtl_math_RoundingMode_Corrected );
			// return makeAny(OUString::createFromAscii(e.print(contextptr).c_str()));
			return makeAny(arrondi);
		} else
		{
			OUString resultat=OUString::createFromAscii(f.print(contextptr).c_str());
			return makeAny(resultat);
		}
	}*/

}

Any CmathOOoCASAddinImpl::subst( Any const & contenu1, Any const & contenu2 , Any const & contenu3)
{
	// giac::context * contextptr=0;
	if (!contenu1.hasValue() || !contenu2.hasValue() || !contenu3.hasValue()) return syntaxErreur();
	OString expression;
	/*gen f;
	gen g;
	gen h;
	f=Cellule2Gen(contenu1);
	g=Cellule2Gen(contenu2);
	h=Cellule2Gen(contenu3);
	OString chaine1=f.print(contextptr).c_str();
	OString chaine2=g.print(contextptr).c_str();
	OString chaine3=h.print(contextptr).c_str();*/
	OString chaine1=Cellule2String(contenu1);
	OString chaine2=Cellule2String(contenu2);
	OString chaine3=Cellule2String(contenu3);
	expression = "subst("+chaine1+","+chaine2+"="+chaine3+")";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}


Any CmathOOoCASAddinImpl::cexact( Any const & contenu)
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::exact(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::csub( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue() || !contenu2.hasValue()) return syntaxErreur();
	gen e;
	try {
        e=Cellule2Gen(contenu1)-Cellule2Gen(contenu2);
		e=giac::simplify(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::cdiv( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue() || !contenu2.hasValue()) return syntaxErreur();
	gen e;
	try {
		e=Cellule2Gen(contenu1)/Cellule2Gen(contenu2);
		e=giac::simplify(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::cpow( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue() || !contenu2.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	try {
		e=giac::pow(Cellule2Gen(contenu1),Cellule2Gen(contenu2),contextptr);
		e=eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}



Any CmathOOoCASAddinImpl::csqrt( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::sqrt(e,contextptr);
		e=eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::csin( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::sin(e,contextptr);
		e=eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::ccos( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::cos(e,contextptr);
		e=eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::ctan( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::tan(e,contextptr);
		e=eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::cln( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::ln(e,contextptr);
		e=eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::cexp( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::exp(e,contextptr);
		e=eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::conj( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::conj(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::arg( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::arg(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::re( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::re(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::im( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::im(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}
 
Any CmathOOoCASAddinImpl::evalc( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	OString chaine=e.print(contextptr).c_str();
	OString expression = "evalc("+chaine+")";
	gen f(expression.getStr(),contextptr);	
	try {
		f=giac::eval(f,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(f);
}

void Somme_Array(gen & e, Sequence< Sequence< Any > > & aArraySeq)
{
	// somme le contenu de la zone de cellules désignée par aArraySeq
	sal_Int32		n1, n2;
	sal_Int32		nE1 = aArraySeq.getLength();
	sal_Int32		nE2;
	// std::cout << "Somme_Array : " << std::endl;
	for( n1 = 0 ; n1 < nE1 ; n1++ )
	{
		const Sequence< Any >	rList = aArraySeq[ n1 ];
		nE2 = rList.getLength();
		const Any*	pList = rList.getConstArray();
		for( n2 = 0 ; n2 < nE2 ; n2++ )
		{
			if (CelluleVide(pList[n2])==true)
			{
				// ne rien faire car cellule vide
				std::cout << "Somme_Array cellule vide : x=" << n1 << ",y=" << n2 << std::endl;
			}
			else
			{
				std::cout << "Somme_Array cellule pleine : x=" << n1 << ",y=" << n2 << ", contenu = " << pList[n2] << std::endl;
				gen temp;
				temp = Cellule2Gen(pList[n2]);
				e+=temp;
			}
			
			
/*			OUString type_name=pList[n2].getValueTypeName();
			// version modifiée de Cellule2Gen pour gérer les cellules vides
			if (type_name==type_double)
			{
				double valeur;
				long partie_entiere;
				pList[n2] >>= valeur;
				partie_entiere=(long)(valeur);
				if (partie_entiere==valeur)
				{
					// retrouver le type int perdu par OOo
					e+=partie_entiere;		
					std::cout << "x:" << n1 << ",y:" << n2 << std::endl;
					std::cout << "int : " << partie_entiere << "," << Cellule2String(pList[n2]) << std::endl;
	
				}
				else
				{
					// non entier
					// chaine=OUString::valueOf(valeur);
					e+=valeur;
					std::cout << "x:" << n1 << ",y:" << n2 << std::endl;
					std::cout << "double : " << valeur << "," << Cellule2String(pList[n2]) << std::endl;
				}
			}
			else
			{	
				if (type_name==type_string)
				{
					OUString chaine;
					pList[n2] >>= chaine;
					OString aOString;
					if (chaine.getLength()!=0)
					{ 
						aOString = OUStringToOString (chaine,RTL_TEXTENCODING_UTF8);
						gen temp(aOString.getStr(),contextptr);
						e+=temp;
						std::cout << "x:" << n1 << ",y:" << n2 << std::endl;
						std::cout << "string : " << aOString << "," << Cellule2String(pList[n2]) << std::endl;
					}
					else
					{ 
						// ne rien faire, la cellule est vide
						// cela peut se produire quand on somme une zone de cellules dont certaines seront amenées
						// à se remplir plus tard.
						// aOString = OUStringToOString (OUString::createFromAscii("nulle"),RTL_TEXTENCODING_UTF8);
						// std::cout << "x:" << n1 << ",y:" << n2 << std::endl;
						// std::cout << "vide " << std::endl;
					}
				}
			}*/		
		}
	}
}


Any CmathOOoCASAddinImpl::cadd(const Sequence< Any > &aValList )
{ 	sal_Int32 n1;
	sal_Int32 nE1 = aValList.getLength();
	Sequence< Sequence< Any > > aArraySeq;
	gen e;
	// giac::context * contextptr=0;
	e=0;
	for( n1 = 0 ; n1 < nE1 ; n1++ )
	{
		try 
		{
			// merci à Niklas Nebel de chez SUN pour ses explications sur ce mode de fonctionnement
			// de OOoCalc qui n'est pas documenté dans le SDK
			if ( aValList[ n1 ] >>= aArraySeq )
			{
				// traiter la zone de cellules contenue dans aArraySeq
				// 
				// std::cout << "type : " << OUStringToOString (aValList[n1].getValueTypeName(),RTL_TEXTENCODING_UTF8) << std::endl;

				Somme_Array(e,aArraySeq);
			}
			else
			{
				// aValList[n1] ne contient qu'une valeur
				// std::cout << "une valeur " << Cellule2String(aValList[ n1 ]) << std::endl;
				// std::cout << "type : " << OUStringToOString (aValList[n1].getValueTypeName(),RTL_TEXTENCODING_UTF8) << std::endl;
				e=e+Cellule2Gen(aValList[ n1 ]);
			}
		} 
		catch (std::runtime_error & erreur) 
		{
			return makeAny(OUString::createFromAscii(erreur.what()));
		}
	}
	e=giac::normal(e,contextptr);
	return Gen2any(e);
}


void Multiplie_Array(gen & e, Sequence< Sequence< Any > > & aArraySeq)
{
	// multiplie le contenu de la zone de cellules désignée par aArraySeq
	sal_Int32		n1, n2;
	sal_Int32		nE1 = aArraySeq.getLength();
	sal_Int32		nE2;
	for( n1 = 0 ; n1 < nE1 ; n1++ )
	{
		const Sequence< Any >	rList = aArraySeq[ n1 ];
		nE2 = rList.getLength();
		const Any*	pList = rList.getConstArray();
		for( n2 = 0 ; n2 < nE2 ; n2++ )
		{
			if (CelluleVide(pList[n2])==true)
			{
				// ne rien faire car cellule vide
				std::cout << "Somme_Array cellule vide : x=" << n1 << ",y=" << n2 << std::endl;
			}
			else
			{
				std::cout << "Somme_Array cellule pleine : x=" << n1 << ",y=" << n2 << ", contenu = " << pList[n2] << std::endl;
				gen temp;
				temp = Cellule2Gen(pList[n2]);
				e=e*temp;
			}
/*

			// version modifiée de Cellule2Gen pour ignorer les cellules vides
			OUString type_name=pList[n2].getValueTypeName();
			if (type_name==type_double)
			{
				double valeur;
				sal_Int32 partie_entiere;
				pList[n2] >>= valeur;
				partie_entiere=(sal_Int32)(valeur);
				if (partie_entiere==valeur)
				{
					// retrouver le type int perdu par OOo
					gen temp((long)partie_entiere);
					e=e*temp;		
				}
				else
				{
					// non entier
					// chaine=OUString::valueOf(valeur);
					gen temp(valeur);
					e=e*temp;
				}
			}
			else
			{	
				if (type_name==OUString::createFromAscii("string"))
				{
					OUString chaine;
					pList[n2] >>= chaine;
					OString aOString;
					if (chaine.getLength()!=0)
					{ 
						aOString = OUStringToOString (chaine,RTL_TEXTENCODING_UTF8);
						gen temp(aOString.getStr(),0);
						e=e*temp;
					}
					else
					{ 
					// ne rien faire, la cellule est vide
					// cela peut se produire quand on multiplie une zone de cellules dont certaines seront amenées
					// à se remplir plus tard.
					// aOString = OUStringToOString (OUString::createFromAscii("nulle"),RTL_TEXTENCODING_UTF8);
					}
				}
			}*/
		}
	}
}


Any CmathOOoCASAddinImpl::cmul(const Sequence< Any >  &aValList )
{ 	sal_Int32 n1;
	sal_Int32 nE1 = aValList.getLength();
	Sequence< Sequence< Any > > aArraySeq;
	gen e;
	// giac::context * contextptr=0;
	e=1;
	for( n1 = 0 ; n1 < nE1 ; n1++ )
	{
		try 
		{
			// merci à Niklas Nebel de chez SUN pour ses explications sur ce mode de fonctionnement
			// de OOoCalc qui n'est pas documenté dans le SDK
			if ( aValList[ n1 ] >>= aArraySeq )
			{
				// traiter la zone de cellules contenue dans aArraySeq
				// 
				Multiplie_Array(e,aArraySeq);
			}
			else
			{
				// aValList[n1] ne contient qu'une valeur
				e=e*Cellule2Gen(aValList[ n1 ]);
			}
		} 
		catch (std::runtime_error & erreur) 
		{
			return makeAny(OUString::createFromAscii(erreur.what()));
		}
	}
	e=giac::normal(e,contextptr);
	return Gen2any(e);
}


// idivis en colonne
Sequence< Sequence< Any > > CmathOOoCASAddinImpl::idivis( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreurMatrix();

	// giac::context * contextptr=0;
	OString entier;
	OString expression;
	OUString type_name=contenu.getValueTypeName();
	entier = Cellule2String( contenu );
	expression="idivis("+entier+")";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		Sequence< Sequence< Any > > sortie;
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=makeAny(OUString::createFromAscii(erreur.what()));
		solList2D[0]=solList1D;
		sortie=solList2D;
		return sortie;
	}
	return Gen2Matrix(e,contextptr);
}



Any CmathOOoCASAddinImpl::deriver( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString chaine1=Cellule2String(contenu1);
	OString expression = "deriver("+chaine1;
	if (contenu2.hasValue())
	{
		OString chaine2=Cellule2String(contenu2);
		expression += ","+chaine2+")";
	}
	else
	{
		expression += ")";
	}
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::formeCanonique( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString chaine1=Cellule2String(contenu1);
	OString expression = "canonical_form("+chaine1;
	if (contenu2.hasValue())
	{
		OString chaine2=Cellule2String(contenu2);
		expression += ","+chaine2+")";
	}
	else
	{
		expression += ")";
	}
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::integrer( Any const & contenu1, Any const & contenu2 , Any const & contenu3, Any const & contenu4)
{
	if (!contenu1.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString expression;
	OString chaine1=Cellule2String(contenu1);
	expression = "integrer("+chaine1;
	if (contenu2.hasValue())
	{
		OString chaine2=Cellule2String(contenu2);
		expression+=","+chaine2;
		if (contenu3.hasValue())
		{
			OString chaine3=Cellule2String(contenu3);
			OString chaine4=Cellule2String(contenu4);
			expression+=","+chaine3+","+chaine4+")";			
		}
		else
		{
			expression+=")";							
		}
	}
	else
	{
		expression += ")";
	}
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::csomme( Any const & contenu1, Any const & contenu2 , Any const & contenu3, Any const & contenu4)
{
	if (!contenu1.hasValue() || !contenu2.hasValue() || !contenu3.hasValue() || !contenu4.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString expression;
	OString chaine1=Cellule2String(contenu1);
	OString chaine2=Cellule2String(contenu2);
	OString chaine3=Cellule2String(contenu3);
	OString chaine4=Cellule2String(contenu4);
	expression = "somme("+chaine1+","+chaine2+","+chaine3+","+chaine4+")";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::cproduit( Any const & contenu1, Any const & contenu2 , Any const & contenu3, Any const & contenu4)
{
	if (!contenu1.hasValue() || !contenu2.hasValue() || !contenu3.hasValue() || !contenu4.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString expression;
	OString chaine1=Cellule2String(contenu1);
	OString chaine2=Cellule2String(contenu2);
	OString chaine3=Cellule2String(contenu3);
	OString chaine4=Cellule2String(contenu4);
	expression = "mul("+chaine1+","+chaine2+","+chaine3+","+chaine4+")";	
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::limite( Any const & contenu1, Any const & contenu2 , Any const & contenu3, Any const & contenu4)
{
	if (!contenu1.hasValue() || !contenu2.hasValue() || !contenu3.hasValue() ) return syntaxErreur();
	// giac::context * contextptr=0;
	OString expression;
	OString chaine1=Cellule2String(contenu1);
	OString chaine2=Cellule2String(contenu2);
	OString chaine3=Cellule2String(contenu3);
	expression = "limite("+chaine1+","+chaine2+","+chaine3;
	if (contenu4.hasValue())
	{
		OString chaine4=Cellule2String(contenu4);
		expression += ","+chaine4+")";
	}
	else
	{
		expression += ")";
	}
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
	//debug 
	// OUString resultat=OUString::createFromAscii("\34"+expression+"\34");
	// return makeAny(resultat);
}


Sequence< Sequence< Any > > CmathOOoCASAddinImpl::makelist( Any const & contenu1, Any const & contenu2, Any const & contenu3, Any const & contenu4, Any const & contenu5 )
{
	if (!contenu1.hasValue() || !contenu2.hasValue() || !contenu3.hasValue() || !contenu4.hasValue()) return syntaxErreurMatrix();
	// giac::context * contextptr=0;
	OString expression;
	OString chaine1=Cellule2String(contenu1);
	OString chaine2=Cellule2String(contenu2);
	OString chaine3=Cellule2String(contenu3);
	OString chaine4=Cellule2String(contenu4);
	expression = "makelist("+chaine2+"->"+chaine1+","+chaine3+","+chaine4;
	if (contenu5.hasValue())
	{
		OString chaine5=Cellule2String(contenu5);
		expression += ","+chaine5+")";
	}
	else
	{
		expression += ")";
	}
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		Sequence< Sequence< Any > > sortie;
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=makeAny(OUString::createFromAscii(erreur.what()));
		solList2D[0]=solList1D;
		sortie=solList2D;
		return sortie;
	}
	return Gen2Matrix(e,contextptr);
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::zeros( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue()) return syntaxErreurMatrix();
	// giac::context * contextptr=0;
	OString chaine1=Cellule2String(contenu1);
	OString expression = "zeros("+chaine1;
	if (contenu2.hasValue())
	{
		OString chaine2=Cellule2String(contenu2);
		expression += ","+chaine2+")";
	}
	else
	{
		expression += ")";
	}
	gen e(expression.getStr(),contextptr);
	Sequence< Sequence< Any > > sortie;
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=makeAny(OUString::createFromAscii(erreur.what()));
		solList2D[0]=solList1D;
		sortie=solList2D;
		return sortie;
	}
	sortie=Gen2Matrix(e,contextptr);
	if (sortie.hasElements()==false)
	{	// pas de solution
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		OUString eAigu((sal_Unicode)0x00E9);
		solList1D[0]=makeAny(OUString::createFromAscii("Pas de z")+eAigu+OUString::createFromAscii("ros"));
		solList2D[0]=solList1D;
		sortie=solList2D;
	}
	return sortie;}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::czeros( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue()) return syntaxErreurMatrix();
	// giac::context * contextptr=0;
	OString chaine1=Cellule2String(contenu1);
	OString expression = "cZeros("+chaine1;
	if (contenu2.hasValue())
	{
		OString chaine2=Cellule2String(contenu2);
		expression += ","+chaine2+")";
	}
	else
	{
		expression += ")";
	}
	gen e(expression.getStr(),contextptr);
	Sequence< Sequence< Any > > sortie;
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=makeAny(OUString::createFromAscii(erreur.what()));
		solList2D[0]=solList1D;
		sortie=solList2D;
		return sortie;
	}
	sortie=Gen2Matrix(e,contextptr);
	if (sortie.hasElements()==false)
	{	// pas de solution
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		OUString eAigu((sal_Unicode)0x00E9);
		solList1D[0]=makeAny(OUString::createFromAscii("Pas de z")+eAigu+OUString::createFromAscii("ros"));
		solList2D[0]=solList1D;
		sortie=solList2D;
	}
	return sortie;
}


Any CmathOOoCASAddinImpl::factorial( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString chaine=Cellule2String(contenu);
	OString expression = "factorial("+chaine+")";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::factoriser( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString chaine=Cellule2String(contenu);
	OString expression = "factoriser("+chaine+")";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::factoriserSurC( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString chaine=Cellule2String(contenu);
	OString expression = "factoriser_sur_C("+chaine+")";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::developper( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString chaine=Cellule2String(contenu);
	OString expression = "developper("+chaine+")";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}


Any CmathOOoCASAddinImpl::iquo( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue() || !contenu2.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	try {
		e=giac::iquo(Cellule2Gen(contenu1),Cellule2Gen(contenu2));
		e=eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::irem( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue() || !contenu2.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString chaine1=Cellule2String(contenu1);
	OString chaine2=Cellule2String(contenu2);
	OString expression = "irem("+chaine1+","+chaine2+")";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::quo( Any const & contenu1, Any const & contenu2 , Any const & contenu3)
{
	if (!contenu1.hasValue() || !contenu2.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString expression;
	gen f;
	gen g;
	gen h;
	f=Cellule2Gen(contenu1);
	OString chaine1=f.print(contextptr).c_str();
	g=Cellule2Gen(contenu2);
	OString chaine2=g.print(contextptr).c_str();
	expression = "quo("+chaine1+","+chaine2;
	if (contenu3.hasValue())
	{
		h=Cellule2Gen(contenu3);
		OString chaine3=h.print(contextptr).c_str();
		expression += ","+chaine3+")";
	}
	else
	{
		expression += ")";
	}
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::rem( Any const & contenu1, Any const & contenu2 , Any const & contenu3)
{
	if (!contenu1.hasValue() || !contenu2.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString expression;
	gen f;
	gen g;
	gen h;
	f=Cellule2Gen(contenu1);
	OString chaine1=f.print(contextptr).c_str();
	g=Cellule2Gen(contenu2);
	OString chaine2=g.print(contextptr).c_str();
	expression = "rem("+chaine1+","+chaine2;
	if (contenu3.hasValue())
	{
		h=Cellule2Gen(contenu3);
		OString chaine3=h.print(contextptr).c_str();
		expression += ","+chaine3+")";
	}
	else
	{
		expression += ")";
	}
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::divis( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreurMatrix();
	// giac::context * contextptr=0;
	OString entier;
	OString expression;
	OUString type_name=contenu.getValueTypeName();
	entier = Cellule2String( contenu );
	expression="divis("+entier+")";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		Sequence< Sequence< Any > > sortie;
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=makeAny(OUString::createFromAscii(erreur.what()));
		solList2D[0]=solList1D;
		sortie=solList2D;
		return sortie;
	}
	return Gen2Matrix(e,contextptr);
}

Any CmathOOoCASAddinImpl::gcd( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue() || !contenu2.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString chaine1=Cellule2String(contenu1);
	OString chaine2=Cellule2String(contenu2);
	OString expression = "gcd("+chaine1+","+chaine2+")";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::lcm( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue() || !contenu2.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString chaine1=Cellule2String(contenu1);
	OString chaine2=Cellule2String(contenu2);
	OString expression = "lcm("+chaine1+","+chaine2+")";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::resoudre( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue()) return syntaxErreurMatrix();
	// giac::context * contextptr=0;
	OString chaine1=Cellule2String(contenu1);
	OString chaine2="";
	if (contenu2.hasValue())
	{
		chaine2=","+Cellule2String(contenu2);
	}
	OString expression = "solve("+chaine1+chaine2+")";
	gen e(expression.getStr(),contextptr);
	Sequence< Sequence< Any > > sortie;
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=makeAny(OUString::createFromAscii(erreur.what()));
		solList2D[0]=solList1D;
		sortie=solList2D;
		return sortie;
	}
	sortie=Gen2Matrix(e,contextptr);
	if (sortie.hasElements()==false)
	{	// pas de solution
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=makeAny(OUString::createFromAscii("Pas de solution"));
		solList2D[0]=solList1D;
		sortie=solList2D;
	}
	return sortie;
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::resoudreDansC( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue()) return syntaxErreurMatrix();
	// giac::context * contextptr=0;
	OString chaine1=Cellule2String(contenu1);
	OString chaine2="";
	if (contenu2.hasValue())
	{
		chaine2=","+Cellule2String(contenu2);
	}
	OString expression = "csolve("+chaine1+chaine2+")";
	gen e(expression.getStr(),contextptr);
	Sequence< Sequence< Any > > sortie;
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=makeAny(OUString::createFromAscii(erreur.what()));
		solList2D[0]=solList1D;
		sortie=solList2D;
		return sortie;
	}
	sortie=Gen2Matrix(e,contextptr);
	if (sortie.hasElements()==false)
	{	// pas de solution
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=makeAny(OUString::createFromAscii("Pas de solution"));
		solList2D[0]=solList1D;
		sortie=solList2D;
	}
	return sortie;
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::resoudreNumerique( Any const & contenu1, Any const & contenu2 , Any const & contenu3, Any const & contenu4)
{
	if (!contenu1.hasValue()) return syntaxErreurMatrix();
	// giac::context * contextptr=0;
	OString expression;
	OString chaine1=Cellule2String(contenu1);
	expression = "fsolve("+chaine1;
	if (contenu2.hasValue())
	{
		OString chaine2=Cellule2String(contenu2);
		expression+=","+chaine2;
		if (contenu3.hasValue())
		{
			OString chaine3=Cellule2String(contenu3);
			expression+=","+chaine3;
			if (contenu4.hasValue())
			{
				OString chaine4=Cellule2String(contenu4);
				expression+=chaine4+")";
			}
			else
			{
				expression+=")";							
			}
		}
		else
		{
			expression+=")";							
		}
	}
	else
	{
		expression += ")";
	}
	gen e(expression.getStr(),contextptr);
	Sequence< Sequence< Any > > sortie;
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=makeAny(OUString::createFromAscii(erreur.what()));
		solList2D[0]=solList1D;
		sortie=solList2D;
		return sortie;
	}
	sortie=Gen2Matrix(e,contextptr);
	if (sortie.hasElements()==false)
	{	// pas de solution
		Sequence< Any >	solList1D(1);
		Sequence< Sequence< Any > > solList2D(1);
		solList1D[0]=makeAny(OUString::createFromAscii("Pas de solution"));
		solList2D[0]=solList1D;
		sortie=solList2D;
	}
	return sortie;
}

Any CmathOOoCASAddinImpl::factoriserEntier( Any const & contenu)
{
	if (!contenu.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString chaine1=Cellule2String(contenu);
	OString expression = "ifactor("+chaine1+")";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::isprime( Any const & contenu)
{
	if (!contenu.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString chaine1=Cellule2String(contenu);
	OString expression = "isprime("+chaine1+")";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	if (e.val==1)
		return makeAny(OUString::createFromAscii("Vrai"));
	else
		return makeAny(OUString::createFromAscii("Faux"));
}


Any CmathOOoCASAddinImpl::carcsin( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::asin(e,contextptr);
		e=eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::carccos( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::acos(e,contextptr);
		e=eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::carctan( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::atan(e,contextptr);
		e=eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::prevprime( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::prevprime(e);
		e=eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::nextprime( Any const & contenu )
{
	if (!contenu.hasValue()) return syntaxErreur();
	gen e;
	// giac::context * contextptr=0;
	e=Cellule2Gen(contenu);
	try {
		e=giac::nextprime(e);
		e=eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::isPseudoPrime( Any const & contenu )
{
	return notYetImplemented();
		// à voir : is_pseudoprime n'est pas connu de giac
		// e=giac::is_pseudoprime(e);
}

Any CmathOOoCASAddinImpl::taylor( Any const & contenu1, Any const & contenu2 , Any const & contenu3)
{
	if (!contenu1.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString expression;
	OString chaine1=Cellule2String(contenu1);
	expression = "taylor("+chaine1;
	if (contenu2.hasValue())
	{
		OString chaine2=Cellule2String(contenu2);
		expression+=","+chaine2;
		if (contenu3.hasValue())
		{
			OString chaine3=Cellule2String(contenu3);
			expression+=","+chaine3+")";			
		}
		else
		{
			expression+=")";							
		}
	}
	else
	{
		expression += ")";
	}
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}



Sequence< Sequence< Any > > CmathOOoCASAddinImpl::iabcuv( Any const & contenu1, Any const & contenu2 , Any const & contenu3)
{
	return notYetImplementedMatrix();
}


Sequence< Sequence< Any > > CmathOOoCASAddinImpl::iegcd( Any const & contenu1, Any const & contenu2 )
{
	return notYetImplementedMatrix();
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::abcuv( Any const & contenu1, Any const & contenu2 , Any const & contenu3 , Any const & contenu4)
{
	return notYetImplementedMatrix();
}


Sequence< Sequence< Any > > CmathOOoCASAddinImpl::egcd( Any const & contenu1, Any const & contenu2 , Any const & contenu3)
{
	return notYetImplementedMatrix();
}


Any CmathOOoCASAddinImpl::fmax( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString chaine1=Cellule2String(contenu1);
	OString expression = "fMax("+chaine1;
	if (contenu2.hasValue())
	{
		OString chaine2=Cellule2String(contenu2);
		expression += ","+chaine2+")";
	}
	else
	{
		expression += ")";
	}
	// renvoyer la première valeur seulement
	// il aurait fallu prévoir le type <Sequence> Any en retour... trop tard
	expression+="[0]";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::fmin( Any const & contenu1, Any const & contenu2 )
{
	if (!contenu1.hasValue()) return syntaxErreur();
	// giac::context * contextptr=0;
	OString chaine1=Cellule2String(contenu1);
	OString expression = "fMin("+chaine1;
	if (contenu2.hasValue())
	{
		OString chaine2=Cellule2String(contenu2);
		expression += ","+chaine2+")";
	}
	else
	{
		expression += ")";
	}
	// renvoyer la première valeur seulement
	// il aurait fallu prévoir le type <Sequence> Any en retour... trop tard
	expression+="[0]";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Any CmathOOoCASAddinImpl::euler( Any const & contenu)
{
	return notYetImplemented();
}

Any CmathOOoCASAddinImpl::latex( Any const & contenu)
{
	return notYetImplemented();
}

Any CmathOOoCASAddinImpl::mathml( Any const & contenu)
{
	return notYetImplemented();
}

Any CmathOOoCASAddinImpl::developperTranscendant( Any const & contenu)
{
	return notYetImplemented();
}

Any CmathOOoCASAddinImpl::ccosh( Any const & contenu)
{
	return notYetImplemented();
}

Any CmathOOoCASAddinImpl::csinh( Any const & contenu)
{
	return notYetImplemented();
}

Any CmathOOoCASAddinImpl::ctanh( Any const & contenu)
{
	return notYetImplemented();
}

Any CmathOOoCASAddinImpl::ccotan( Any const & contenu)
{
	return notYetImplemented();
}

Any CmathOOoCASAddinImpl::carccosh( Any const & contenu)
{
	return notYetImplemented();
}

Any CmathOOoCASAddinImpl::carcsinh( Any const & contenu)
{
	return notYetImplemented();
}

Any CmathOOoCASAddinImpl::carctanh( Any const & contenu)
{
	return notYetImplemented();
}

Any CmathOOoCASAddinImpl::carccotan( Any const & contenu)
{
	return notYetImplemented();
}

Any CmathOOoCASAddinImpl::dsolve( Any const & contenu1, Any const & contenu2 , Any const & contenu3, Any const & contenu4)
{
	return notYetImplemented();
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::egv(const Sequence< Sequence< Any > > &aArrayCell )
{ 
	gen e;
	try {
                e=giac::megv(Cellules2Gen(aArrayCell),contextptr);
	} catch (std::runtime_error & erreur){
		return OUString2AnyMatrix(OUString::createFromAscii(erreur.what()));
	}
	return Gen2Matrix(e,contextptr);
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::eigenvals(const Sequence< Sequence< Any > > &aArrayCell )
{
	matrice m=Cellules2Gen(aArrayCell);
	gen temp=m;
	OString chaine=Gen2String(temp);
	OString expression = "eigenvals("+chaine+")";
	gen e(expression.getStr(),contextptr);
	try {
		e=giac::eval(e,contextptr);
	} catch (std::runtime_error & erreur){
		return OUString2AnyMatrix(OUString::createFromAscii(erreur.what()));
	}
	return Gen2Matrix(e,contextptr);
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::linsolve(const Sequence< Sequence< Any > > &aArrayCell )
{ 	return notYetImplementedMatrix();
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::matinv(const Sequence< Sequence< Any > > &aArrayCell )	
{ 	
	gen e;
	try {
		// matrice m;
		// m=Cellules2Gen(aArrayCell);
                e=giac::minv(Cellules2Gen(aArrayCell),contextptr);
		//e=giac::simplify(e,contextptr);
	} catch (std::runtime_error & erreur){
		return OUString2AnyMatrix(OUString::createFromAscii(erreur.what()));
	}
	return Gen2Matrix(e,contextptr);
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::matadd(const Sequence< Sequence< Any > > &aArrayCell1, const Sequence< Sequence< Any > > &aArrayCell2 )
{ 	
	gen e;
	try {
                e=Cellules2Gen(aArrayCell1)+Cellules2Gen(aArrayCell2);
		e=giac::simplify(e,contextptr);
	} catch (std::runtime_error & erreur){
		return OUString2AnyMatrix(OUString::createFromAscii(erreur.what()));
	}
	return Gen2Matrix(e,contextptr);
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::matsub(const Sequence< Sequence< Any > > &aArrayCell1, const Sequence< Sequence< Any > > &aArrayCell2 )
{ 	
	gen e;
	try {
                e=Cellules2Gen(aArrayCell1)-Cellules2Gen(aArrayCell2);
		e=giac::simplify(e,contextptr);
	} catch (std::runtime_error & erreur){
		return OUString2AnyMatrix(OUString::createFromAscii(erreur.what()));
	}
	return Gen2Matrix(e,contextptr);
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::matmult(const Sequence< Sequence< Any > > &aArrayCell1, const Sequence< Sequence< Any > > &aArrayCell2 )	
{ 	
	gen e;
	try {
                e=giac::mmult(Cellules2Gen(aArrayCell1),Cellules2Gen(aArrayCell2));
		//e=giac::simplify(e,contextptr);
	} catch (std::runtime_error & erreur){
		return OUString2AnyMatrix(OUString::createFromAscii(erreur.what()));
	}
	return Gen2Matrix(e,contextptr);
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::matpow(const Sequence< Sequence< Any > > &aArrayCell, const Any & contenu)	
{
	gen e;
	gen exposant;	
	exposant=Cellule2Gen(contenu);
	try {
		// ! giac::matpow élève à la puissance en Jordanisant.
                e=giac::pow(Cellules2Gen(aArrayCell),exposant,contextptr);

	} catch (std::runtime_error & erreur){
		return OUString2AnyMatrix(OUString::createFromAscii(erreur.what()));
	}
	return Gen2Matrix(e,contextptr);
}

Any CmathOOoCASAddinImpl::det(const Sequence< Sequence< Any > > &aArrayCell )
{
	gen e;
	try {
                e=giac::mdet(Cellules2Gen(aArrayCell),contextptr);
	} catch (std::runtime_error & erreur){
		return makeAny(OUString::createFromAscii(erreur.what()));
	}
	return Gen2any(e);
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::cas1(const Sequence< Any > &aValList)	
{ 	
	// fonctionnement interne à OOoBasic : permet d'initialiser l'aide "?command"
	// dans la boite de dialogue CAS.
	if (!nbAides){
		OUString chemin;
		aValList[0] >>= chemin;
		OUString language;
		aValList[1] >>= language;
		sal_Int32 langue;
		langue = string2lang((OUStringToOString(language,RTL_TEXTENCODING_UTF8)).getStr());
		OUString cheminAides;
		OUString cheminSynonymes;
		cheminAides=chemin+OUString::createFromAscii("aide_cas");
		cheminSynonymes=chemin+language+OUString::createFromAscii("/keywords");
		std::string helpFile=(OUStringToOString(cheminAides,RTL_TEXTENCODING_UTF8)).getStr();
		std::string synonymsFile=(OUStringToOString(cheminSynonymes,RTL_TEXTENCODING_UTF8)).getStr();
		// ajoute les synonymes, code de giac/global.cc/add_language modifié ligne 2190 (giac 0.8.6) -> ligne 3213 (giac 1.5.0)
		giac::language(langue,contextptr);
     	lexer_localization_vector().push_back(langue); // 1 pour fr
		// fonction suivante modifiée pour tenir compte du chemin d'accès de keywords dans l'extension OOo
		cmath_update_lexer_localization(lexer_localization_vector(),lexer_localization_map(),back_lexer_localization_map(), synonymsFile);
		// ajoute les aides
		giac::readhelp(*vector_aide_ptr(),helpFile.c_str(),(int&) nbAides,true);
		// (*giac::vector_aide_ptr)=giac::readhelp(helpFile.c_str(),(int&) nbAides);
		if (vector_aide_ptr()){
			// add locale command description
			// int count;
			// string filename=giac_aide_dir()+find_doc_prefix(i)+"aide_cas";
			// readhelp(*vector_aide_ptr,filename.c_str(),count,true);
			// add synonyms
			multimap<string,localized_string>::iterator it,backend=back_lexer_localization_map().end(),itend;
			vector<aide>::iterator jt = vector_aide_ptr()->begin(),jtend=vector_aide_ptr()->end();
			for (;jt!=jtend;++jt){
				it=back_lexer_localization_map().find(jt->cmd_name);
				itend=back_lexer_localization_map().upper_bound(jt->cmd_name);
				if (it!=backend){
				 for (;it!=itend;++it){
			      if (it->second.language==langue)
				jt->synonymes.push_back(it->second);
			    }
			  }
			}
			sal_Int32 s = int(vector_aide_ptr()->size());
			for (sal_Int32 j=0;j<s;++j){
			  aide a=(*vector_aide_ptr())[j];
			  it=back_lexer_localization_map().find(a.cmd_name);
			  itend=back_lexer_localization_map().upper_bound(a.cmd_name);
			  if (it!=backend){
			    for (;it!=itend;++it){
			      if (it->second.language==langue){
					a.cmd_name=it->second.chaine;
					a.language=it->second.language;
					vector_aide_ptr()->push_back(a);
			      }
			    }
			  }
			}
			nbSynonymes=vector_aide_ptr()->size()-s;
			//	cerr << "Added " << vector_aide_ptr->size()-s << " synonyms" << endl;
			//	sort(vector_aide_ptr->begin(),vector_aide_ptr->end(),alpha_order);
			//	update_completions();
		}
	}
	OUString sNbAides;
	OUString sNbSynonymes;
	sNbAides=OUString::number(nbAides);
	sNbSynonymes=OUString::number(nbSynonymes);
	Sequence< Any >	solList1D(2);
	Sequence< Sequence< Any > > solList2D(1);
	solList1D[0]=makeAny(sNbAides);
	solList1D[1]=makeAny(sNbSynonymes);
	solList2D[0]=solList1D;
	return solList2D;
}

// repris de giac/input_lexer.cc ligne 7848 (giac 0.8.6) ou giac/global.cc (giac 1.5.0) pour changer le chemin d'accès au fichier keywords
void CmathOOoCASAddinImpl::cmath_update_lexer_localization(const std::vector<int> & v,std::map<std::string,std::string> &lexer_map,std::multimap<std::string,giac::localized_string> &back_lexer_map, std::string &synonymsFile)
{
      lexer_map.clear();
      back_lexer_map.clear();
      int s=v.size();
      for (int i=0;i<s;++i){
	int lang=v[i];
	if (lang>=1 && lang<=4){
	  // std::string doc=find_doc_prefix(lang);
	  // std::string file=giac::giac_aide_dir()+doc+"keywords";
	  std::string giac_kw,local_kw;
	  size_t l;
	  char * line = (char *)malloc(1024);
	  ifstream f(synonymsFile.c_str());
	  if (f.good()){
	    cerr << "// Using keyword file " << synonymsFile << endl;
	    for (;;){
	      f.getline(line,1023,'\n');
	      l=strlen(line);
	      if (f.eof()){
		f.close();
		break;
	      }
	      if (l>3 && line[0]!='#'){
		if (line[l-1]=='\n')
		  --l;
		// read giac keyword
		size_t j;
		giac_kw="";
		for (j=0;j<l;++j){
		  if (line[j]==' ')
		    break;
		  giac_kw += line[j];
		}
		// read corresponding local keywords
		local_kw="";
		for (++j;j<l;++j){
		  if (line[j]==' '){
		    if (!local_kw.empty()){
		      lexer_map[local_kw]=giac_kw;
		      back_lexer_map.insert(pair<string,localized_string>(giac_kw,localized_string(lang,local_kw)));
		    }
		    local_kw="";
		  }
		  else
		    local_kw += line[j];
		}
		if (!local_kw.empty()){
		  lexer_map[local_kw]=giac_kw;
		  back_lexer_map.insert(pair<string,localized_string>(giac_kw,localized_string(lang,local_kw)));
		}
	      }
	    }
	    free(line);
	  } // if (f)
	  else
	    cerr << "// Unable to find keyword file " << synonymsFile << endl;
	}
  }
}



Sequence< Sequence< Any > > CmathOOoCASAddinImpl::cas2(const Sequence< Any > &aValList)
	
{ 	return notYetImplementedMatrix();
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::cas3(const Sequence< Any > &aValList)
	
{ 	return notYetImplementedMatrix();
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::cas4(const Sequence< Any > &aValList)
	
{ 	return notYetImplementedMatrix();
}

Sequence< Sequence< Any > > CmathOOoCASAddinImpl::cas5(const Sequence< Any > &aValList)
	
{ 	return notYetImplementedMatrix();
}

 
// ------------------------------------------------------------------------------------------
 
// XAddIn
 
OUString SAL_CALL CmathOOoCASAddinImpl::getProgrammaticFuntionName( const OUString& aDisplayName )
{
    //  not used by calc
    //  (but should be implemented for other uses of the AddIn service)
    return OUString();
}
 
OUString SAL_CALL CmathOOoCASAddinImpl::getDisplayFunctionName( const OUString& aProgrammaticName )
{

	// tableau des noms de fonctions localisées
	// indice 0 = ProgrammaticName
	// indice 1 = french DisplayFunctionName
	// indice 2 = english DisplayFunctionName
    // indice 3 = deutsch DisplayFunctionName
    // indice ?? = your language
	// if you want to add your own language, send me your translation at : cdeval@free.fr
		
	OUString DisplayFunctionName[][10]={
		{
		OUString::createFromAscii("casFonction"),
		OUString::createFromAscii("casFonction"),
		OUString::createFromAscii("casFunction"),
		OUString::createFromAscii("casFonction"),
		OUString::createFromAscii("casFonction"),
		OUString::createFromAscii("casFonction"),
		OUString::createFromAscii("casFonction"),
		OUString::createFromAscii("casFonction"),
		OUString::createFromAscii("casFonction"),
		OUString::createFromAscii("casFonction")

		},
		{
		OUString::createFromAscii("casCompil"),
		OUString::createFromAscii("casCompil"),
		OUString::createFromAscii("casCompil"),
		OUString::createFromAscii("casCompil"),
		OUString::createFromAscii("casCompil"),
		OUString::createFromAscii("casCompil"),
		OUString::createFromAscii("casCompil"),
		OUString::createFromAscii("casCompil"),
		OUString::createFromAscii("casCompil"),
		OUString::createFromAscii("casCompil")
		},
		{
		OUString::createFromAscii("cas"),
		OUString::createFromAscii("cas"),
		OUString::createFromAscii("cas"),
		OUString::createFromAscii("cas"),
		OUString::createFromAscii("cas"),
		OUString::createFromAscii("cas"),
		OUString::createFromAscii("cas"),
		OUString::createFromAscii("cas"),
		OUString::createFromAscii("cas"),
		OUString::createFromAscii("cas")
		},
		{
		OUString::createFromAscii("cabs"),
		OUString::createFromAscii("cabs"),
		OUString::createFromAscii("cabs"),
		OUString::createFromAscii("cabs"), // to translate in deutsch
		OUString::createFromAscii("cabs"),
		OUString::createFromAscii("cabs"),
		OUString::createFromAscii("cabs"),
		OUString::createFromAscii("cabs"),
		OUString::createFromAscii("cabs"),
		OUString::createFromAscii("cabs")
		},
		{
		OUString::createFromAscii("simplifier"),
		OUString::createFromAscii("simplifier"),
		OUString::createFromAscii("simplify"),
		OUString::createFromAscii("simplify"), // to translate in deutsch
		OUString::createFromAscii("simplify"),
		OUString::createFromAscii("simplify"),
		OUString::createFromAscii("simplify"),
		OUString::createFromAscii("simplify"),
		OUString::createFromAscii("simplify"),
		OUString::createFromAscii("simplify")
		},
		{
		OUString::createFromAscii("re"),
		OUString::createFromAscii("creel"),
		OUString::createFromAscii("creal"),
		OUString::createFromAscii("creal"), // to translate in deutsch
		OUString::createFromAscii("creal"),
		OUString::createFromAscii("creal"),
		OUString::createFromAscii("creal"),
		OUString::createFromAscii("creal"),
		OUString::createFromAscii("creal"),
		OUString::createFromAscii("creal")
		},
		{
		OUString::createFromAscii("im"),
		OUString::createFromAscii("cimaginaire"),
		OUString::createFromAscii("cimaginary"),
		OUString::createFromAscii("cimaginary"), // to translate in deutsch
		OUString::createFromAscii("cimaginary"),
		OUString::createFromAscii("cimaginary"),
		OUString::createFromAscii("cimaginary"),
		OUString::createFromAscii("cimaginary"),
		OUString::createFromAscii("cimaginary"),
		OUString::createFromAscii("cimaginary")
		},
		{
		OUString::createFromAscii("cadd"),
		OUString::createFromAscii("csomme"),
		OUString::createFromAscii("csum"),
		OUString::createFromAscii("csum"), // to translate in deutsch
		OUString::createFromAscii("csum"),
		OUString::createFromAscii("csum"),
		OUString::createFromAscii("csum"),
		OUString::createFromAscii("csum"),
		OUString::createFromAscii("csum"),
		OUString::createFromAscii("csum")
		},
		{
		OUString::createFromAscii("cmul"),
		OUString::createFromAscii("cproduit"),
		OUString::createFromAscii("cproduct"),
		OUString::createFromAscii("cproduct"), // to translate in deutsch
		OUString::createFromAscii("cproduct"),
		OUString::createFromAscii("cproduct"),
		OUString::createFromAscii("cproduct"),
		OUString::createFromAscii("cproduct"),
		OUString::createFromAscii("cproduct"),
		OUString::createFromAscii("cproduct")
		},
		{
		OUString::createFromAscii("csub"),
		OUString::createFromAscii("csoustraction"),
		OUString::createFromAscii("csubtraction"),
		OUString::createFromAscii("csubtraction"), // to translate in deutsch
		OUString::createFromAscii("csubtraction"),
		OUString::createFromAscii("csubtraction"),
		OUString::createFromAscii("csubtraction"),
		OUString::createFromAscii("csubtraction"),
		OUString::createFromAscii("csubtraction"),
		OUString::createFromAscii("csubtraction")
		},
		{
		OUString::createFromAscii("cdiv"),
		OUString::createFromAscii("cquotient"),
		OUString::createFromAscii("cquotient"),
		OUString::createFromAscii("cquotient"), // to translate in deutsch
		OUString::createFromAscii("cquotient"),
		OUString::createFromAscii("cquotient"),
		OUString::createFromAscii("cquotient"),
		OUString::createFromAscii("cquotient"),
		OUString::createFromAscii("cquotient"),
		OUString::createFromAscii("cquotient")
		},
		{
		OUString::createFromAscii("cpow"),
		OUString::createFromAscii("cpuissance"),
		OUString::createFromAscii("cpower"),
		OUString::createFromAscii("cpower"), // to translate in deutsch
		OUString::createFromAscii("cpower"),
		OUString::createFromAscii("cpower"),
		OUString::createFromAscii("cpower"),
		OUString::createFromAscii("cpower"),
		OUString::createFromAscii("cpower"),
		OUString::createFromAscii("cpower")
		},
		{
		OUString::createFromAscii("csqrt"),
		OUString::createFromAscii("cracine"),
		OUString::createFromAscii("csqrt"),
		OUString::createFromAscii("csqrt"), // to translate in deutsch
		OUString::createFromAscii("csqrt"),
		OUString::createFromAscii("csqrt"),
		OUString::createFromAscii("csqrt"),
		OUString::createFromAscii("csqrt"),
		OUString::createFromAscii("csqrt"),
		OUString::createFromAscii("csqrt")
		},
		{
		OUString::createFromAscii("csin"),
		OUString::createFromAscii("csin"),
		OUString::createFromAscii("csin"),
		OUString::createFromAscii("csin"), // to translate in deutsch
		OUString::createFromAscii("csin"),
		OUString::createFromAscii("csin"),
		OUString::createFromAscii("csin"),
		OUString::createFromAscii("csin"),
		OUString::createFromAscii("csin"),
		OUString::createFromAscii("csin")
		},
		{
		OUString::createFromAscii("ccos"),
		OUString::createFromAscii("ccos"),
		OUString::createFromAscii("ccos"),
		OUString::createFromAscii("ccos"), // to translate in deutsch
		OUString::createFromAscii("ccos"),
		OUString::createFromAscii("ccos"),
		OUString::createFromAscii("ccos"),
		OUString::createFromAscii("ccos"),
		OUString::createFromAscii("ccos"),
		OUString::createFromAscii("ccos")
		},
		{
		OUString::createFromAscii("ctan"),
		OUString::createFromAscii("ctan"),
		OUString::createFromAscii("ctan"),
		OUString::createFromAscii("ctan"), // to translate in deutsch
		OUString::createFromAscii("ctan"),
		OUString::createFromAscii("ctan"),
		OUString::createFromAscii("ctan"),
		OUString::createFromAscii("ctan"),
		OUString::createFromAscii("ctan"),
		OUString::createFromAscii("ctan")
		},
		{
		OUString::createFromAscii("cln"),
		OUString::createFromAscii("cln"),
		OUString::createFromAscii("cln"),
		OUString::createFromAscii("cln"), // to translate in deutsch
		OUString::createFromAscii("cln"),
		OUString::createFromAscii("cln"),
		OUString::createFromAscii("cln"),
		OUString::createFromAscii("cln"),
		OUString::createFromAscii("cln"),
		OUString::createFromAscii("cln")
		},
		{
		OUString::createFromAscii("cexp"),
		OUString::createFromAscii("cexp"),
		OUString::createFromAscii("cexp"),
		OUString::createFromAscii("cexp"), // to translate in deutsch
		OUString::createFromAscii("cexp"),
		OUString::createFromAscii("cexp"),
		OUString::createFromAscii("cexp"),
		OUString::createFromAscii("cexp"),
		OUString::createFromAscii("cexp"),
		OUString::createFromAscii("cexp")
		},
		{
		OUString::createFromAscii("conj"),
		OUString::createFromAscii("cconjugue"),
		OUString::createFromAscii("cconjugate"),
		OUString::createFromAscii("cconjugate"), // to translate in deutsch
		OUString::createFromAscii("cconjugate"),
		OUString::createFromAscii("cconjugate"),
		OUString::createFromAscii("cconjugate"),
		OUString::createFromAscii("cconjugate"),
		OUString::createFromAscii("cconjugate"),
		OUString::createFromAscii("cconjugate")
		},
		{
		OUString::createFromAscii("arg"),
		OUString::createFromAscii("cargument"),
		OUString::createFromAscii("cargument"),
		OUString::createFromAscii("cargument"), // to translate in deutsch
		OUString::createFromAscii("cargument"),
		OUString::createFromAscii("cargument"),
		OUString::createFromAscii("cargument"),
		OUString::createFromAscii("cargument"),
		OUString::createFromAscii("cargument"),
		OUString::createFromAscii("cargument")
		},
		{
		OUString::createFromAscii("evalc"),
		OUString::createFromAscii("evalc"),
		OUString::createFromAscii("evalc"),
		OUString::createFromAscii("evalc"), // to translate in deutsch
		OUString::createFromAscii("evalc"),
		OUString::createFromAscii("evalc"),
		OUString::createFromAscii("evalc"),
		OUString::createFromAscii("evalc"),
		OUString::createFromAscii("evalc"),
		OUString::createFromAscii("evalc")
		},
		{
		OUString::createFromAscii("subst"),
		OUString::createFromAscii("substituer"),
		OUString::createFromAscii("csubstitute"),
		OUString::createFromAscii("csubstitute"), // to translate in deutsch
		OUString::createFromAscii("csubstitute"),
		OUString::createFromAscii("csubstitute"),
		OUString::createFromAscii("csubstitute"),
		OUString::createFromAscii("csubstitute"),
		OUString::createFromAscii("csubstitute"),
		OUString::createFromAscii("csubstitute")
		},
		{
		OUString::createFromAscii("evalf"),
		OUString::createFromAscii("evaluer"),
		OUString::createFromAscii("evaluate"),
		OUString::createFromAscii("evaluate"), // to translate in deutsch
		OUString::createFromAscii("evaluate"),
		OUString::createFromAscii("evaluate"),
		OUString::createFromAscii("evaluate"),
		OUString::createFromAscii("evaluate"),
		OUString::createFromAscii("evaluate"),
		OUString::createFromAscii("evaluate")
		},
		{
		OUString::createFromAscii("cexact"),
		OUString::createFromAscii("cexact"),
		OUString::createFromAscii("cexact"),
		OUString::createFromAscii("cexact"), // to translate in deutsch
		OUString::createFromAscii("cexact"),
		OUString::createFromAscii("cexact"),
		OUString::createFromAscii("cexact"),
		OUString::createFromAscii("cexact"),
		OUString::createFromAscii("cexact"),
		OUString::createFromAscii("cexact")
		},
		{
		OUString::createFromAscii("idivis"),
		OUString::createFromAscii("diviseursEntier"),
		OUString::createFromAscii("integerDivisors"),
		OUString::createFromAscii("integerDivisors"), // to translate in deutsch
		OUString::createFromAscii("integerDivisors"),
		OUString::createFromAscii("integerDivisors"),
		OUString::createFromAscii("integerDivisors"),
		OUString::createFromAscii("integerDivisors"),
		OUString::createFromAscii("integerDivisors"),
		OUString::createFromAscii("integerDivisors")
		},
		{
		OUString::createFromAscii("deriver"),
		OUString::createFromAscii("deriver"),
		OUString::createFromAscii("differentiate"),
		OUString::createFromAscii("differentiate"), // to translate in deutsch
		OUString::createFromAscii("differentiate"),
		OUString::createFromAscii("differentiate"),
		OUString::createFromAscii("differentiate"),
		OUString::createFromAscii("differentiate"),
		OUString::createFromAscii("differentiate"),
		OUString::createFromAscii("differentiate")
		},
		{
		OUString::createFromAscii("integrer"),
		OUString::createFromAscii("integrer"),
		OUString::createFromAscii("integrate"),
		OUString::createFromAscii("integrate"), // to translate in deutsch
		OUString::createFromAscii("integrate"),
		OUString::createFromAscii("integrate"),
		OUString::createFromAscii("integrate"),
		OUString::createFromAscii("integrate"),
		OUString::createFromAscii("integrate"),
		OUString::createFromAscii("integrate")
		},
		{
		OUString::createFromAscii("csomme"),
		OUString::createFromAscii("sommeDiscrete"),
		OUString::createFromAscii("discretSum"),
		OUString::createFromAscii("discretSum"), // to translate in deutsch
		OUString::createFromAscii("discretSum"),
		OUString::createFromAscii("discretSum"),
		OUString::createFromAscii("discretSum"),
		OUString::createFromAscii("discretSum"),
		OUString::createFromAscii("discretSum"),
		OUString::createFromAscii("discretSum")
		},
		{
		OUString::createFromAscii("cproduit"),
		OUString::createFromAscii("produitDiscret"),
		OUString::createFromAscii("discretProduct"),
		OUString::createFromAscii("discretProduct"), // to translate in deutsch
		OUString::createFromAscii("discretProduct"),
		OUString::createFromAscii("discretProduct"),
		OUString::createFromAscii("discretProduct"),
		OUString::createFromAscii("discretProduct"),
		OUString::createFromAscii("discretProduct"),
		OUString::createFromAscii("discretProduct")
		},
		{
		OUString::createFromAscii("limite"),
		OUString::createFromAscii("limite"),
		OUString::createFromAscii("limit"),
		OUString::createFromAscii("limit"), // to translate in deutsch
		OUString::createFromAscii("limit"),
		OUString::createFromAscii("limit"),
		OUString::createFromAscii("limit"),
		OUString::createFromAscii("limit"),
		OUString::createFromAscii("limit"),
		OUString::createFromAscii("limit")
		},
		{
		OUString::createFromAscii("makelist"),
		OUString::createFromAscii("tableauValeurs"),
		OUString::createFromAscii("table"),
		OUString::createFromAscii("table"), // to translate in deutsch
		OUString::createFromAscii("table"),
		OUString::createFromAscii("table"),
		OUString::createFromAscii("table"),
		OUString::createFromAscii("table"),
		OUString::createFromAscii("table"),
		OUString::createFromAscii("table")
		},
		{
		OUString::createFromAscii("czeros"),
		OUString::createFromAscii("czeros"),
		OUString::createFromAscii("czeros"),
		OUString::createFromAscii("czeros"), // to translate in deutsch
		OUString::createFromAscii("czeros"),
		OUString::createFromAscii("czeros"),
		OUString::createFromAscii("czeros"),
		OUString::createFromAscii("czeros"),
		OUString::createFromAscii("czeros"),
		OUString::createFromAscii("czeros")
		},
		{
		OUString::createFromAscii("zeros"),
		OUString::createFromAscii("zeros"),
		OUString::createFromAscii("zeros"),
		OUString::createFromAscii("zeros"), // to translate in deutsch
		OUString::createFromAscii("zeros"),
		OUString::createFromAscii("zeros"),
		OUString::createFromAscii("zeros"),
		OUString::createFromAscii("zeros"),
		OUString::createFromAscii("zeros"),
		OUString::createFromAscii("zeros")
		},
		{
		OUString::createFromAscii("factorial"),
		OUString::createFromAscii("cfact"),
		OUString::createFromAscii("cfact"),
		OUString::createFromAscii("cfact"), // to translate in deutsch
		OUString::createFromAscii("cfact"),
		OUString::createFromAscii("cfact"),
		OUString::createFromAscii("cfact"),
		OUString::createFromAscii("cfact"),
		OUString::createFromAscii("cfact"),
		OUString::createFromAscii("cfact")
		},
		{
		OUString::createFromAscii("factoriser"),
		OUString::createFromAscii("factoriser"),
		OUString::createFromAscii("factorize"),
		OUString::createFromAscii("factorize"), // to translate in deutsch
		OUString::createFromAscii("factorize"),
		OUString::createFromAscii("factorize"),
		OUString::createFromAscii("factorize"),
		OUString::createFromAscii("factorize"),
		OUString::createFromAscii("factorize"),
		OUString::createFromAscii("factorize")
		},
		{
		OUString::createFromAscii("factoriserSurC"),
		OUString::createFromAscii("factoriserSurC"),
		OUString::createFromAscii("factorizeOverC"),
		OUString::createFromAscii("factorizeOverC"), // to translate in deutsch
		OUString::createFromAscii("factorizeOverC"),
		OUString::createFromAscii("factorizeOverC"),
		OUString::createFromAscii("factorizeOverC"),
		OUString::createFromAscii("factorizeOverC"),
		OUString::createFromAscii("factorizeOverC"),
		OUString::createFromAscii("factorizeOverC")
		},
		{
		OUString::createFromAscii("developper"),
		OUString::createFromAscii("developper"),
		OUString::createFromAscii("develop"),
		OUString::createFromAscii("develop"), // to translate in deutsch
		OUString::createFromAscii("develop"),
		OUString::createFromAscii("develop"),
		OUString::createFromAscii("develop"),
		OUString::createFromAscii("develop"),
		OUString::createFromAscii("develop"),
		OUString::createFromAscii("develop")
		},
		{
		OUString::createFromAscii("iquo"),
		OUString::createFromAscii("quotientEntiers"),
		OUString::createFromAscii("integersQuotient"),
		OUString::createFromAscii("integersQuotient"), // to translate in deutsch
		OUString::createFromAscii("integersQuotient"),
		OUString::createFromAscii("integersQuotient"),
		OUString::createFromAscii("integersQuotient"),
		OUString::createFromAscii("integersQuotient"),
		OUString::createFromAscii("integersQuotient"),
		OUString::createFromAscii("integersQuotient")
		},
		{
		OUString::createFromAscii("irem"),
		OUString::createFromAscii("resteEntiers"),
		OUString::createFromAscii("integersReminder"),
		OUString::createFromAscii("integersReminder"), // to translate in deutsch
		OUString::createFromAscii("integersReminder"),
		OUString::createFromAscii("integersReminder"),
		OUString::createFromAscii("integersReminder"),
		OUString::createFromAscii("integersReminder"),
		OUString::createFromAscii("integersReminder"),
		OUString::createFromAscii("integersReminder")
		},
		{
		OUString::createFromAscii("quo"),
		OUString::createFromAscii("quotientPolynomes"),
		OUString::createFromAscii("polynomialsQuotient"),
		OUString::createFromAscii("polynomialsQuotient"), // to translate in deutsch
		OUString::createFromAscii("polynomialsQuotient"),
		OUString::createFromAscii("polynomialsQuotient"),
		OUString::createFromAscii("polynomialsQuotient"),
		OUString::createFromAscii("polynomialsQuotient"),
		OUString::createFromAscii("polynomialsQuotient"),
		OUString::createFromAscii("polynomialsQuotient")
		},
		{
		OUString::createFromAscii("rem"),
		OUString::createFromAscii("restePolynomes"),
		OUString::createFromAscii("polynomialsReminder"),
		OUString::createFromAscii("polynomialsReminder"), // to translate in deutsch
		OUString::createFromAscii("polynomialsReminder"),
		OUString::createFromAscii("polynomialsReminder"),
		OUString::createFromAscii("polynomialsReminder"),
		OUString::createFromAscii("polynomialsReminder"),
		OUString::createFromAscii("polynomialsReminder"),
		OUString::createFromAscii("polynomialsReminder")
		},
		{
		OUString::createFromAscii("resoudre"),
		OUString::createFromAscii("resoudre"),
		OUString::createFromAscii("solve"),
		OUString::createFromAscii("solve"), // to translate in deutsch
		OUString::createFromAscii("solve"),
		OUString::createFromAscii("solve"),
		OUString::createFromAscii("solve"),
		OUString::createFromAscii("solve"),
		OUString::createFromAscii("solve"),
		OUString::createFromAscii("solve")
		},
		{
		OUString::createFromAscii("resoudreDansC"),
		OUString::createFromAscii("resoudreDansC"),
		OUString::createFromAscii("solveInC"),
		OUString::createFromAscii("solveInC"), // to translate in deutsch
		OUString::createFromAscii("solveInC"),
		OUString::createFromAscii("solveInC"),
		OUString::createFromAscii("solveInC"),
		OUString::createFromAscii("solveInC"),
		OUString::createFromAscii("solveInC"),
		OUString::createFromAscii("solveInC")
		},
		{
		OUString::createFromAscii("resoudreNumerique"),
		OUString::createFromAscii("resoudreNumerique"),
		OUString::createFromAscii("numericalSolve"),
		OUString::createFromAscii("numericalSolve"), // to translate in deutsch
		OUString::createFromAscii("numericalSolve"),
		OUString::createFromAscii("numericalSolve"),
		OUString::createFromAscii("numericalSolve"),
		OUString::createFromAscii("numericalSolve"),
		OUString::createFromAscii("numericalSolve"),
		OUString::createFromAscii("numericalSolve")
		},
		{
		OUString::createFromAscii("gcd"),
		OUString::createFromAscii("cpgcd"),
		OUString::createFromAscii("cgcd"),
		OUString::createFromAscii("cgcd"), // to translate in deutsch
		OUString::createFromAscii("cgcd"),
		OUString::createFromAscii("cgcd"),
		OUString::createFromAscii("cgcd"),
		OUString::createFromAscii("cgcd"),
		OUString::createFromAscii("cgcd"),
		OUString::createFromAscii("cgcd")
		},
		{
		OUString::createFromAscii("lcm"),
		OUString::createFromAscii("cppcm"),
		OUString::createFromAscii("clcm"),
		OUString::createFromAscii("clcm"), // to translate in deutsch
		OUString::createFromAscii("clcm"),
		OUString::createFromAscii("clcm"),
		OUString::createFromAscii("clcm"),
		OUString::createFromAscii("clcm"),
		OUString::createFromAscii("clcm"),
		OUString::createFromAscii("clcm")
		},
		{
		OUString::createFromAscii("formeCanonique"),
		OUString::createFromAscii("formeCanonique"),
		OUString::createFromAscii("canonicalForm"),
		OUString::createFromAscii("canonicalForm"), // to translate in deutsch
		OUString::createFromAscii("canonicalForm"),
		OUString::createFromAscii("canonicalForm"),
		OUString::createFromAscii("canonicalForm"),
		OUString::createFromAscii("canonicalForm"),
		OUString::createFromAscii("canonicalForm"),
		OUString::createFromAscii("canonicalForm")
		},
		{
		OUString::createFromAscii("factoriserEntier"),
		OUString::createFromAscii("factoriserEntier"),
		OUString::createFromAscii("integerFactorize"),
		OUString::createFromAscii("integerFactorize"), // to translate in deutsch
		OUString::createFromAscii("integerFactorize"),
		OUString::createFromAscii("integerFactorize"),
		OUString::createFromAscii("integerFactorize"),
		OUString::createFromAscii("integerFactorize"),
		OUString::createFromAscii("integerFactorize"),
		OUString::createFromAscii("integerFactorize")
		},
		{
		OUString::createFromAscii("isprime"),
		OUString::createFromAscii("estPremier"),
		OUString::createFromAscii("isPrime"),
		OUString::createFromAscii("isPrime"), // to translate in deutsch
		OUString::createFromAscii("isPrime"),
		OUString::createFromAscii("isPrime"),
		OUString::createFromAscii("isPrime"),
		OUString::createFromAscii("isPrime"),
		OUString::createFromAscii("isPrime"),
		OUString::createFromAscii("isPrime")
		},
		{
		OUString::createFromAscii("carcsin"),
		OUString::createFromAscii("carcsin"),
		OUString::createFromAscii("carcsin"),
		OUString::createFromAscii("carcsin"),
		OUString::createFromAscii("carcsin"),
		OUString::createFromAscii("carcsin"),
		OUString::createFromAscii("carcsin"),
		OUString::createFromAscii("carcsin"),
		OUString::createFromAscii("carcsin"),
		OUString::createFromAscii("carcsin")
		},
		{
		OUString::createFromAscii("carccos"),
		OUString::createFromAscii("carccos"),
		OUString::createFromAscii("carccos"),
		OUString::createFromAscii("carccos"),
		OUString::createFromAscii("carccos"),
		OUString::createFromAscii("carccos"),
		OUString::createFromAscii("carccos"),
		OUString::createFromAscii("carccos"),
		OUString::createFromAscii("carccos"),
		OUString::createFromAscii("carccos")
		},
		{
		OUString::createFromAscii("carctan"),
		OUString::createFromAscii("carctan"),
		OUString::createFromAscii("carctan"),
		OUString::createFromAscii("carctan"),
		OUString::createFromAscii("carctan"),
		OUString::createFromAscii("carctan"),
		OUString::createFromAscii("carctan"),
		OUString::createFromAscii("carctan"),
		OUString::createFromAscii("carctan"),
		OUString::createFromAscii("carctan")
		},
		{
		OUString::createFromAscii("divis"),
		OUString::createFromAscii("diviseursPolynome"),
		OUString::createFromAscii("polynomialDivisors"),
		OUString::createFromAscii("polynomialDivisors"), // to translate in deutsch
		OUString::createFromAscii("polynomialDivisors"),
		OUString::createFromAscii("polynomialDivisors"),
		OUString::createFromAscii("polynomialDivisors"),
		OUString::createFromAscii("polynomialDivisors"),
		OUString::createFromAscii("polynomialDivisors"),
		OUString::createFromAscii("polynomialDivisors")
		},
		{
		OUString::createFromAscii("taylor"),
		OUString::createFromAscii("taylor"),
		OUString::createFromAscii("taylor"),
		OUString::createFromAscii("taylor"),
		OUString::createFromAscii("taylor"),
		OUString::createFromAscii("taylor"),
		OUString::createFromAscii("taylor"),
		OUString::createFromAscii("taylor"),
		OUString::createFromAscii("taylor"),
		OUString::createFromAscii("taylor")
		},
		{
		OUString::createFromAscii("prevprime"),
		OUString::createFromAscii("premierPrecedent"),
		OUString::createFromAscii("previousPrime"),
		OUString::createFromAscii("previousPrime"), // to translate in deutsch
		OUString::createFromAscii("previousPrime"),
		OUString::createFromAscii("previousPrime"),
		OUString::createFromAscii("previousPrime"),
		OUString::createFromAscii("previousPrime"),
		OUString::createFromAscii("previousPrime"),
		OUString::createFromAscii("previousPrime")
		},
		{
		OUString::createFromAscii("nextprime"),
		OUString::createFromAscii("premierSuivant"),
		OUString::createFromAscii("nextPrime"),
		OUString::createFromAscii("nextPrime"), // to translate in deutsch
		OUString::createFromAscii("nextPrime"),
		OUString::createFromAscii("nextPrime"),
		OUString::createFromAscii("nextPrime"),
		OUString::createFromAscii("nextPrime"),
		OUString::createFromAscii("nextPrime"),
		OUString::createFromAscii("nextPrime")
		},
		{
		OUString::createFromAscii("isPseudoPrime"),
		OUString::createFromAscii("estPseudoPremier"),
		OUString::createFromAscii("isPseudoPrime"),
		OUString::createFromAscii("isPseudoPrime"), // to translate in deutsch
		OUString::createFromAscii("isPseudoPrime"),
		OUString::createFromAscii("isPseudoPrime"),
		OUString::createFromAscii("isPseudoPrime"),
		OUString::createFromAscii("isPseudoPrime"),
		OUString::createFromAscii("isPseudoPrime"),
		OUString::createFromAscii("isPseudoPrime")
		},
		{
		OUString::createFromAscii("euler"),
		OUString::createFromAscii("euler"),
		OUString::createFromAscii("euler"),
		OUString::createFromAscii("euler"), // to translate in deutsch
		OUString::createFromAscii("euler"),
		OUString::createFromAscii("euler"),
		OUString::createFromAscii("euler"),
		OUString::createFromAscii("euler"),
		OUString::createFromAscii("euler"),
		OUString::createFromAscii("euler")
		},
		{
		OUString::createFromAscii("latex"),
		OUString::createFromAscii("latex"),
		OUString::createFromAscii("latex"),
		OUString::createFromAscii("latex"),
		OUString::createFromAscii("latex"),
		OUString::createFromAscii("latex"),
		OUString::createFromAscii("latex"),
		OUString::createFromAscii("latex"),
		OUString::createFromAscii("latex"),
		OUString::createFromAscii("latex")
		},
		{
		OUString::createFromAscii("mathml"),
		OUString::createFromAscii("mathml"),
		OUString::createFromAscii("mathml"),
		OUString::createFromAscii("mathml"),
		OUString::createFromAscii("mathml"),
		OUString::createFromAscii("mathml"),
		OUString::createFromAscii("mathml"),
		OUString::createFromAscii("mathml"),
		OUString::createFromAscii("mathml"),
		OUString::createFromAscii("mathml")
		},
		{
		OUString::createFromAscii("developperTranscendant"),
		OUString::createFromAscii("developperTranscendant"),
		OUString::createFromAscii("transcendentalDevelop"),
		OUString::createFromAscii("transcendentalDevelop"), // to translate in deutsch
		OUString::createFromAscii("transcendentalDevelop"),
		OUString::createFromAscii("transcendentalDevelop"),
		OUString::createFromAscii("transcendentalDevelop"),
		OUString::createFromAscii("transcendentalDevelop"),
		OUString::createFromAscii("transcendentalDevelop"),
		OUString::createFromAscii("transcendentalDevelop")
		},
		{
		OUString::createFromAscii("ccosh"),
		OUString::createFromAscii("ccosh"),
		OUString::createFromAscii("ccosh"),
		OUString::createFromAscii("ccosh"),
		OUString::createFromAscii("ccosh"),
		OUString::createFromAscii("ccosh"),
		OUString::createFromAscii("ccosh"),
		OUString::createFromAscii("ccosh"),
		OUString::createFromAscii("ccosh"),
		OUString::createFromAscii("ccosh")
		},
		{
		OUString::createFromAscii("csinh"),
		OUString::createFromAscii("csinh"),
		OUString::createFromAscii("csinh"),
		OUString::createFromAscii("csinh"),
		OUString::createFromAscii("csinh"),
		OUString::createFromAscii("csinh"),
		OUString::createFromAscii("csinh"),
		OUString::createFromAscii("csinh"),
		OUString::createFromAscii("csinh"),
		OUString::createFromAscii("csinh")
		},
		{
		OUString::createFromAscii("ctanh"),
		OUString::createFromAscii("ctanh"),
		OUString::createFromAscii("ctanh"),
		OUString::createFromAscii("ctanh"),
		OUString::createFromAscii("ctanh"),
		OUString::createFromAscii("ctanh"),
		OUString::createFromAscii("ctanh"),
		OUString::createFromAscii("ctanh"),
		OUString::createFromAscii("ctanh"),
		OUString::createFromAscii("ctanh")
		},
		{
		OUString::createFromAscii("ccotan"),
		OUString::createFromAscii("ccotan"),
		OUString::createFromAscii("ccotan"),
		OUString::createFromAscii("ccotan"),
		OUString::createFromAscii("ccotan"),
		OUString::createFromAscii("ccotan"),
		OUString::createFromAscii("ccotan"),
		OUString::createFromAscii("ccotan"),
		OUString::createFromAscii("ccotan"),
		OUString::createFromAscii("ccotan")
		},
		{
		OUString::createFromAscii("carccosh"),
		OUString::createFromAscii("carccosh"),
		OUString::createFromAscii("carccosh"),
		OUString::createFromAscii("carccosh"),
		OUString::createFromAscii("carccosh"),
		OUString::createFromAscii("carccosh"),
		OUString::createFromAscii("carccosh"),
		OUString::createFromAscii("carccosh"),
		OUString::createFromAscii("carccosh"),
		OUString::createFromAscii("carccosh")
		},
		{
		OUString::createFromAscii("carcsinh"),
		OUString::createFromAscii("carcsinh"),
		OUString::createFromAscii("carcsinh"),
		OUString::createFromAscii("carcsinh"),
		OUString::createFromAscii("carcsinh"),
		OUString::createFromAscii("carcsinh"),
		OUString::createFromAscii("carcsinh"),
		OUString::createFromAscii("carcsinh"),
		OUString::createFromAscii("carcsinh"),
		OUString::createFromAscii("carcsinh")
		},
		{
		OUString::createFromAscii("carctanh"),
		OUString::createFromAscii("carctanh"),
		OUString::createFromAscii("carctanh"),
		OUString::createFromAscii("carctanh"),
		OUString::createFromAscii("carctanh"),
		OUString::createFromAscii("carctanh"),
		OUString::createFromAscii("carctanh"),
		OUString::createFromAscii("carctanh"),
		OUString::createFromAscii("carctanh"),
		OUString::createFromAscii("carctanh")
		},
		{
		OUString::createFromAscii("carccotan"),
		OUString::createFromAscii("carccotan"),
		OUString::createFromAscii("carccotan"),
		OUString::createFromAscii("carccotan"),
		OUString::createFromAscii("carccotan"),
		OUString::createFromAscii("carccotan"),
		OUString::createFromAscii("carccotan"),
		OUString::createFromAscii("carccotan"),
		OUString::createFromAscii("carccotan"),
		OUString::createFromAscii("carccotan")
		},
		{
		OUString::createFromAscii("iegcd"),
		OUString::createFromAscii("BezoutEntiers"),
		OUString::createFromAscii("integersBezout"),
		OUString::createFromAscii("integersBezout"), // to translate in deutsch
		OUString::createFromAscii("integersBezout"),
		OUString::createFromAscii("integersBezout"),
		OUString::createFromAscii("integersBezout"),
		OUString::createFromAscii("integersBezout"),
		OUString::createFromAscii("integersBezout"),
		OUString::createFromAscii("integersBezout")
		},
		{
		OUString::createFromAscii("iabcuv"),
		OUString::createFromAscii("abcuvEntiers"),
		OUString::createFromAscii("integersabcuv"),
		OUString::createFromAscii("integersabcuv"), // to translate in deutsch
		OUString::createFromAscii("integersabcuv"),
		OUString::createFromAscii("integersabcuv"),
		OUString::createFromAscii("integersabcuv"),
		OUString::createFromAscii("integersabcuv"),
		OUString::createFromAscii("integersabcuv"),
		OUString::createFromAscii("integersabcuv")
		},
		{
		OUString::createFromAscii("egcd"),
		OUString::createFromAscii("BezoutPolynomes"),
		OUString::createFromAscii("polynomialsBezout"),
		OUString::createFromAscii("polynomialsBezout"), // to translate in deutsch
		OUString::createFromAscii("polynomialsBezout"),
		OUString::createFromAscii("polynomialsBezout"),
		OUString::createFromAscii("polynomialsBezout"),
		OUString::createFromAscii("polynomialsBezout"),
		OUString::createFromAscii("polynomialsBezout"),
		OUString::createFromAscii("polynomialsBezout")
		},
		{
		OUString::createFromAscii("abcuv"),
		OUString::createFromAscii("abcuvPolynomes"),
		OUString::createFromAscii("polynomialsabcuv"),
		OUString::createFromAscii("polynomialsabcuv"), // to translate in deutsch
		OUString::createFromAscii("polynomialsabcuv"),
		OUString::createFromAscii("polynomialsabcuv"),
		OUString::createFromAscii("polynomialsabcuv"),
		OUString::createFromAscii("polynomialsabcuv"),
		OUString::createFromAscii("polynomialsabcuv"),
		OUString::createFromAscii("polynomialsabcuv")
		},
		{
		OUString::createFromAscii("fmax"),
		OUString::createFromAscii("fmax"),
		OUString::createFromAscii("fmax"),
		OUString::createFromAscii("fmax"), // to translate in deutsch
		OUString::createFromAscii("fmax"),
		OUString::createFromAscii("fmax"),
		OUString::createFromAscii("fmax"),
		OUString::createFromAscii("fmax"),
		OUString::createFromAscii("fmax"),
		OUString::createFromAscii("fmax")
		},
		{
		OUString::createFromAscii("fmin"),
		OUString::createFromAscii("fmin"),
		OUString::createFromAscii("fmin"),
		OUString::createFromAscii("fmin"), // to translate in deutsch
		OUString::createFromAscii("fmin"),
		OUString::createFromAscii("fmin"),
		OUString::createFromAscii("fmin"),
		OUString::createFromAscii("fmin"),
		OUString::createFromAscii("fmin"),
		OUString::createFromAscii("fmin")
		},
		{
		OUString::createFromAscii("linsolve"),
		OUString::createFromAscii("systemeEquations"),
		OUString::createFromAscii("linsolve"),
		OUString::createFromAscii("linsolve"), // to translate in deutsch
		OUString::createFromAscii("linsolve"),
		OUString::createFromAscii("linsolve"),
		OUString::createFromAscii("linsolve"),
		OUString::createFromAscii("linsolve"),
		OUString::createFromAscii("linsolve"),
		OUString::createFromAscii("linsolve")
		},
		{
		OUString::createFromAscii("egv"),
		OUString::createFromAscii("vecteursPropres"),
		OUString::createFromAscii("eigenVectors"),
		OUString::createFromAscii("eigenVectors"), // to translate in deutsch
		OUString::createFromAscii("eigenVectors"),
		OUString::createFromAscii("eigenVectors"),
		OUString::createFromAscii("eigenVectors"),
		OUString::createFromAscii("eigenVectors"),
		OUString::createFromAscii("eigenVectors"),
		OUString::createFromAscii("eigenVectors")
		},
		{
		OUString::createFromAscii("eigenvals"),
		OUString::createFromAscii("valeursPropres"),
		OUString::createFromAscii("eigenVals"),
		OUString::createFromAscii("eigenVals"), // to translate in deutsch
		OUString::createFromAscii("eigenVals"),
		OUString::createFromAscii("eigenVals"),
		OUString::createFromAscii("eigenVals"),
		OUString::createFromAscii("eigenVals"),
		OUString::createFromAscii("eigenVals"),
		OUString::createFromAscii("eigenVals")
		},
		{
		OUString::createFromAscii("matinv"),
		OUString::createFromAscii("cinversemat"),
		OUString::createFromAscii("cminverse"),
		OUString::createFromAscii("cminverse"), // to translate in deutsch
		OUString::createFromAscii("cminverse"),
		OUString::createFromAscii("cminverse"),
		OUString::createFromAscii("cminverse"),
		OUString::createFromAscii("cminverse"),
		OUString::createFromAscii("cminverse"),
		OUString::createFromAscii("cminverse")
		},
		{
		OUString::createFromAscii("matadd"),
		OUString::createFromAscii("csommemat"),
		OUString::createFromAscii("cmsum"),
		OUString::createFromAscii("cmsum"), // to translate in deutsch
		OUString::createFromAscii("cmsum"),
		OUString::createFromAscii("cmsum"),
		OUString::createFromAscii("cmsum"),
		OUString::createFromAscii("cmsum"),
		OUString::createFromAscii("cmsum"),
		OUString::createFromAscii("cmsum")
		},
		{
		OUString::createFromAscii("matsub"),
		OUString::createFromAscii("csoustractionmat"),
		OUString::createFromAscii("cmsub"),
		OUString::createFromAscii("cmsub"), // to translate in deutsch
		OUString::createFromAscii("cmsub"),
		OUString::createFromAscii("cmsub"),
		OUString::createFromAscii("cmsub"),
		OUString::createFromAscii("cmsub"),
		OUString::createFromAscii("cmsub"),
		OUString::createFromAscii("cmsub")
		},
		{
		OUString::createFromAscii("matmult"),
		OUString::createFromAscii("cproduitmat"),
		OUString::createFromAscii("cmmult"),
		OUString::createFromAscii("cmmult"), // to translate in deutsch
		OUString::createFromAscii("cmmult"),
		OUString::createFromAscii("cmmult"),
		OUString::createFromAscii("cmmult"),
		OUString::createFromAscii("cmmult"),
		OUString::createFromAscii("cmmult"),
		OUString::createFromAscii("cmmult")
		},
		{
		OUString::createFromAscii("matpow"),
		OUString::createFromAscii("cpuissancemat"),
		OUString::createFromAscii("cmpower"),
		OUString::createFromAscii("cmpower"), // to translate in deutsch
		OUString::createFromAscii("cmpower"),
		OUString::createFromAscii("cmpower"),
		OUString::createFromAscii("cmpower"),
		OUString::createFromAscii("cmpower"),
		OUString::createFromAscii("cmpower"),
		OUString::createFromAscii("cmpower")
		},
		{
		OUString::createFromAscii("det"),
		OUString::createFromAscii("cdetermat"),
		OUString::createFromAscii("cmdeterm"),
		OUString::createFromAscii("cmdeterm"), // to translate in deutsch
		OUString::createFromAscii("cmdeterm"),
		OUString::createFromAscii("cmdeterm"),
		OUString::createFromAscii("cmdeterm"),
		OUString::createFromAscii("cmdeterm"),
		OUString::createFromAscii("cmdeterm"),
		OUString::createFromAscii("cmdeterm")
		},
		{
		OUString::createFromAscii("dsolve"),
		OUString::createFromAscii("resoudreED"),
		OUString::createFromAscii("desolve"),
		OUString::createFromAscii("desolve"), // to translate in deutsch
		OUString::createFromAscii("desolve"),
		OUString::createFromAscii("desolve"),
		OUString::createFromAscii("desolve"),
		OUString::createFromAscii("desolve"),
		OUString::createFromAscii("desolve"),
		OUString::createFromAscii("desolve")
		},
		{
		OUString::createFromAscii("cas1"),
		OUString::createFromAscii("cas1"),
		OUString::createFromAscii("cas1"),
		OUString::createFromAscii("cas1"),
		OUString::createFromAscii("cas1"),
		OUString::createFromAscii("cas1"),
		OUString::createFromAscii("cas1"),
		OUString::createFromAscii("cas1"),
		OUString::createFromAscii("cas1"),
		OUString::createFromAscii("cas1")
		},
		{
		OUString::createFromAscii("cas2"),
		OUString::createFromAscii("cas2"),
		OUString::createFromAscii("cas2"),
		OUString::createFromAscii("cas2"),
		OUString::createFromAscii("cas2"),
		OUString::createFromAscii("cas2"),
		OUString::createFromAscii("cas2"),
		OUString::createFromAscii("cas2"),
		OUString::createFromAscii("cas2"),
		OUString::createFromAscii("cas2")
		},
		{
		OUString::createFromAscii("cas3"),
		OUString::createFromAscii("cas3"),
		OUString::createFromAscii("cas3"),
		OUString::createFromAscii("cas3"),
		OUString::createFromAscii("cas3"),
		OUString::createFromAscii("cas3"),
		OUString::createFromAscii("cas3"),
		OUString::createFromAscii("cas3"),
		OUString::createFromAscii("cas3"),
		OUString::createFromAscii("cas3")
		},
		{
		OUString::createFromAscii("cas4"),
		OUString::createFromAscii("cas4"),
		OUString::createFromAscii("cas4"),
		OUString::createFromAscii("cas4"),
		OUString::createFromAscii("cas4"),
		OUString::createFromAscii("cas4"),
		OUString::createFromAscii("cas4"),
		OUString::createFromAscii("cas4"),
		OUString::createFromAscii("cas4"),
		OUString::createFromAscii("cas4")
		},
		{
		OUString::createFromAscii("cas5"),
		OUString::createFromAscii("cas5"),
		OUString::createFromAscii("cas5"),
		OUString::createFromAscii("cas5"),
		OUString::createFromAscii("cas5"),
		OUString::createFromAscii("cas5"),
		OUString::createFromAscii("cas5"),
		OUString::createFromAscii("cas5"),
		OUString::createFromAscii("cas5"),
		OUString::createFromAscii("cas5")
		},
		{ // garde fou. Normalement, on ne renvoie jamais cet élément.
		OUString::createFromAscii("internalError"),
		OUString::createFromAscii("erreurInterne"),
		OUString::createFromAscii("internalError"),
		OUString::createFromAscii("internalError"), // to translate in deutsch
		OUString::createFromAscii("internalError"),
		OUString::createFromAscii("internalError"),
		OUString::createFromAscii("internalError"),
		OUString::createFromAscii("internalError"),
		OUString::createFromAscii("internalError"),
		OUString::createFromAscii("internalError")
		} };
	sal_Int32 i=0;
	sal_Int32 lang=0;
	if (aFuncLoc.Language.equalsAscii("fr"))
	{
		lang=1;
	}
	else if (aFuncLoc.Language.equalsAscii("de"))
	{	
		lang=3;
	}
        else
        {
		lang=2;          
        }
	while (i< nNumFunctions && !aProgrammaticName.equals(DisplayFunctionName[i][0])){
		i++;
	}
	return DisplayFunctionName[i][lang];
}
 
OUString SAL_CALL CmathOOoCASAddinImpl::getFunctionDescription( const OUString& aProgrammaticName )
{
    // caractères français accentués
    OUString eAigu((sal_Unicode)0x00E9); 
    OUString EAigu((sal_Unicode)0x00C9); 
    OUString aGrave((sal_Unicode)0x00E0);
    OUString eGrave((sal_Unicode)0x00E8);
    OUString oCirconflexe((sal_Unicode)0x00F4);
    OUString oTrema((sal_Unicode)0x00F6);
    OUString uTrema((sal_Unicode)0x00FC);
    OUString aTrema((sal_Unicode)0x00E4);

	// tableau des noms de fonctions localisées
	// indice 0 = ProgrammaticName
	// indice 1 = french FunctionDescription
	// indice 2 = english FunctionDescription
        // indice 3 = deutsch FunctionDescription
        // indice ?? = your language
	// if you want to add your own language, send me your translation at : cdeval@free.fr
		
	OUString FunctionDescription[][10]={
		{
		OUString::createFromAscii("casFonction"),
		EAigu+OUString::createFromAscii("value une expression math")+eAigu+OUString::createFromAscii("matique (syntaxe Giac)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Auswerten eines mathematischen Ausdrucks (Giac Syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)")
		},
		{
		OUString::createFromAscii("casCompil"),
		EAigu+OUString::createFromAscii("value une expression math")+eAigu+OUString::createFromAscii("matique (syntaxe Giac)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Auswerten eines mathematischen Ausdrucks (Giac Syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)")
		},
		{
		OUString::createFromAscii("cas"),
		EAigu+OUString::createFromAscii("value une expression math")+eAigu+OUString::createFromAscii("matique (syntaxe Giac)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
                OUString::createFromAscii("Auswerten eines mathematischen Ausdrucks (Giac Syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)"),
		OUString::createFromAscii("Evaluate a mathematical expression (Giac syntax)")
		},
		{
		OUString::createFromAscii("cabs"),
		OUString::createFromAscii("Calcule la valeur absolue ou le module d'un nombre"),
		OUString::createFromAscii("Returns the absolute value or the norm of its argument"),
		OUString::createFromAscii("Returns the absolute value or the norm of its argument"), // to translate in deutsch
		OUString::createFromAscii("Returns the absolute value or the norm of its argument"),
		OUString::createFromAscii("Returns the absolute value or the norm of its argument"),
		OUString::createFromAscii("Returns the absolute value or the norm of its argument"),
		OUString::createFromAscii("Returns the absolute value or the norm of its argument"),
		OUString::createFromAscii("Returns the absolute value or the norm of its argument"),
		OUString::createFromAscii("Returns the absolute value or the norm of its argument")
		},
		{
		OUString::createFromAscii("simplifier"),
		OUString::createFromAscii("Simplifie une expression math")+eAigu+OUString::createFromAscii("matique"),
		OUString::createFromAscii("Simplifies a mathematical expression"),
		OUString::createFromAscii("Simplifies a mathematical expression"), // to translate in deutsch
		OUString::createFromAscii("Simplifies a mathematical expression"),
		OUString::createFromAscii("Simplifies a mathematical expression"),
		OUString::createFromAscii("Simplifies a mathematical expression"),
		OUString::createFromAscii("Simplifies a mathematical expression"),
		OUString::createFromAscii("Simplifies a mathematical expression"),
		OUString::createFromAscii("Simplifies a mathematical expression")
		},
		{
		OUString::createFromAscii("re"),
		OUString::createFromAscii("Renvoie la partie r")+eAigu+OUString::createFromAscii("elle d'un nombre complexe"),
		OUString::createFromAscii("Returns the real part of a complex number"),
		OUString::createFromAscii("Returns the real part of a complex number"), // to translate in deutsch
		OUString::createFromAscii("Returns the real part of a complex number"),
		OUString::createFromAscii("Returns the real part of a complex number"),
		OUString::createFromAscii("Returns the real part of a complex number"),
		OUString::createFromAscii("Returns the real part of a complex number"),
		OUString::createFromAscii("Returns the real part of a complex number"),
		OUString::createFromAscii("Returns the real part of a complex number")
		},
		{
		OUString::createFromAscii("im"),
		OUString::createFromAscii("Renvoie la partie imaginaire d'un nombre complexe"),
		OUString::createFromAscii("Returns the imaginary part of a complex number"),
		OUString::createFromAscii("Returns the imaginary part of a complex number"), // to translate in deutsch
		OUString::createFromAscii("Returns the imaginary part of a complex number"),
		OUString::createFromAscii("Returns the imaginary part of a complex number"),
		OUString::createFromAscii("Returns the imaginary part of a complex number"),
		OUString::createFromAscii("Returns the imaginary part of a complex number"),
		OUString::createFromAscii("Returns the imaginary part of a complex number"),
		OUString::createFromAscii("Returns the imaginary part of a complex number")
		},
		{
		OUString::createFromAscii("cadd"),
		OUString::createFromAscii("Additionne des expressions math")+eAigu+OUString::createFromAscii("matiques"),
		OUString::createFromAscii("Adds mathematical expressions"),
		OUString::createFromAscii("Adds mathematical expressions"), // to translate in deutsch
		OUString::createFromAscii("Adds mathematical expressions"),
		OUString::createFromAscii("Adds mathematical expressions"),
		OUString::createFromAscii("Adds mathematical expressions"),
		OUString::createFromAscii("Adds mathematical expressions"),
		OUString::createFromAscii("Adds mathematical expressions"),
		OUString::createFromAscii("Adds mathematical expressions")
		},
		{
		OUString::createFromAscii("cmul"),
		OUString::createFromAscii("Multiplie des expressions math")+eAigu+OUString::createFromAscii("matiques"),
		OUString::createFromAscii("Multiplies mathematical expressions"),
		OUString::createFromAscii("Multiplies mathematical expressions"), // to translate in deutsch
		OUString::createFromAscii("Multiplies mathematical expressions"),
		OUString::createFromAscii("Multiplies mathematical expressions"),
		OUString::createFromAscii("Multiplies mathematical expressions"),
		OUString::createFromAscii("Multiplies mathematical expressions"),
		OUString::createFromAscii("Multiplies mathematical expressions"),
		OUString::createFromAscii("Multiplies mathematical expressions")
		},
		{
		OUString::createFromAscii("csub"),
		OUString::createFromAscii("Soustrait des expressions math")+eAigu+OUString::createFromAscii("matiques"),
		OUString::createFromAscii("Substracts mathematical expressions"),
		OUString::createFromAscii("Substracts mathematical expressions"), // to translate in deutsch
		OUString::createFromAscii("Substracts mathematical expressions"),
		OUString::createFromAscii("Substracts mathematical expressions"),
		OUString::createFromAscii("Substracts mathematical expressions"),
		OUString::createFromAscii("Substracts mathematical expressions"),
		OUString::createFromAscii("Substracts mathematical expressions"),
		OUString::createFromAscii("Substracts mathematical expressions")
		},
		{
		OUString::createFromAscii("cdiv"),
		OUString::createFromAscii("Divise des expressions math")+eAigu+OUString::createFromAscii("matiques"),
		OUString::createFromAscii("Divides mathematical expressions"),
		OUString::createFromAscii("Divides mathematical expressions"), // to translate in deutsch
		OUString::createFromAscii("Divides mathematical expressions"),
		OUString::createFromAscii("Divides mathematical expressions"),
		OUString::createFromAscii("Divides mathematical expressions"),
		OUString::createFromAscii("Divides mathematical expressions"),
		OUString::createFromAscii("Divides mathematical expressions"),
		OUString::createFromAscii("Divides mathematical expressions")
		},
		{
		OUString::createFromAscii("cpow"),
		OUString::createFromAscii("El")+eGrave+OUString::createFromAscii("ve ")+aGrave+OUString::createFromAscii(" la puissance"),
		OUString::createFromAscii("Raise to power"),
		OUString::createFromAscii("Raise to power"), // to translate in deutsch
		OUString::createFromAscii("Raise to power"),
		OUString::createFromAscii("Raise to power"),
		OUString::createFromAscii("Raise to power"),
		OUString::createFromAscii("Raise to power"),
		OUString::createFromAscii("Raise to power"),
		OUString::createFromAscii("Raise to power")
		},
		{
		OUString::createFromAscii("csqrt"),
		OUString::createFromAscii("Renvoie la racine carree d'une expression math")+eAigu+OUString::createFromAscii("matique"),
		OUString::createFromAscii("Returns square root"),
		OUString::createFromAscii("Returns square root"), // to translate in deutsch
		OUString::createFromAscii("Returns square root"),
		OUString::createFromAscii("Returns square root"),
		OUString::createFromAscii("Returns square root"),
		OUString::createFromAscii("Returns square root"),
		OUString::createFromAscii("Returns square root"),
		OUString::createFromAscii("Returns square root")
		},
		{
		OUString::createFromAscii("csin"),
		OUString::createFromAscii("Renvoie le sinus d'une expression math")+eAigu+OUString::createFromAscii("matique"),
		OUString::createFromAscii("Returns the sinus of a mathematical expression"),
		OUString::createFromAscii("Returns the sinus of a mathematical expression"), // to translate in deutsch
		OUString::createFromAscii("Returns the sinus of a mathematical expression"),
		OUString::createFromAscii("Returns the sinus of a mathematical expression"),
		OUString::createFromAscii("Returns the sinus of a mathematical expression"),
		OUString::createFromAscii("Returns the sinus of a mathematical expression"),
		OUString::createFromAscii("Returns the sinus of a mathematical expression"),
		OUString::createFromAscii("Returns the sinus of a mathematical expression")
		},
		{
		OUString::createFromAscii("ccos"),
		OUString::createFromAscii("Renvoie le cosinus d'une expression math")+eAigu+OUString::createFromAscii("matique"),
		OUString::createFromAscii("Returns the cosinus of a mathematical expression"),
		OUString::createFromAscii("Returns the cosinus of a mathematical expression"), // to translate in deutsch
		OUString::createFromAscii("Returns the cosinus of a mathematical expression"),
		OUString::createFromAscii("Returns the cosinus of a mathematical expression"),
		OUString::createFromAscii("Returns the cosinus of a mathematical expression"),
		OUString::createFromAscii("Returns the cosinus of a mathematical expression"),
		OUString::createFromAscii("Returns the cosinus of a mathematical expression"),
		OUString::createFromAscii("Returns the cosinus of a mathematical expression")
		},
		{
		OUString::createFromAscii("ctan"),
		OUString::createFromAscii("Renvoie la tangente d'une expression math")+eAigu+OUString::createFromAscii("matique"),
		OUString::createFromAscii("Returns the tangent of a mathematical expression"),
		OUString::createFromAscii("Returns the tangent of a mathematical expression"), // to translate in deutsch
		OUString::createFromAscii("Returns the tangent of a mathematical expression"),
		OUString::createFromAscii("Returns the tangent of a mathematical expression"),
		OUString::createFromAscii("Returns the tangent of a mathematical expression"),
		OUString::createFromAscii("Returns the tangent of a mathematical expression"),
		OUString::createFromAscii("Returns the tangent of a mathematical expression"),
		OUString::createFromAscii("Returns the tangent of a mathematical expression")
		},
		{
		OUString::createFromAscii("cln"),
		OUString::createFromAscii("Renvoie le logarithme n")+eAigu+OUString::createFromAscii("p")+eAigu+OUString::createFromAscii("rien d'une expression math")+eAigu+OUString::createFromAscii("matique"),
		OUString::createFromAscii("Returns the natural logarithm of a mathematical expression"),
		OUString::createFromAscii("Returns the natural logarithm of a mathematical expression"), // to translate in deutsch
		OUString::createFromAscii("Returns the natural logarithm of a mathematical expression"),
		OUString::createFromAscii("Returns the natural logarithm of a mathematical expression"),
		OUString::createFromAscii("Returns the natural logarithm of a mathematical expression"),
		OUString::createFromAscii("Returns the natural logarithm of a mathematical expression"),
		OUString::createFromAscii("Returns the natural logarithm of a mathematical expression"),
		OUString::createFromAscii("Returns the natural logarithm of a mathematical expression")
		},
		{
		OUString::createFromAscii("cexp"),
		OUString::createFromAscii("Renvoie l'exponentielle d'une expression math")+eAigu+OUString::createFromAscii("matique"),
		OUString::createFromAscii("Returns the exponential of a mathematical expression"),
		OUString::createFromAscii("Returns the exponential of a mathematical expression"), // to translate in deutsch
		OUString::createFromAscii("Returns the exponential of a mathematical expression"),
		OUString::createFromAscii("Returns the exponential of a mathematical expression"),
		OUString::createFromAscii("Returns the exponential of a mathematical expression"),
		OUString::createFromAscii("Returns the exponential of a mathematical expression"),
		OUString::createFromAscii("Returns the exponential of a mathematical expression"),
		OUString::createFromAscii("Returns the exponential of a mathematical expression")
		},
		{
		OUString::createFromAscii("conj"),
		OUString::createFromAscii("Renvoie le conjugu")+eAigu+OUString::createFromAscii(" d'un nombre complexe"),
		OUString::createFromAscii("Returns the conjugate of a complex number"),
		OUString::createFromAscii("Returns the conjugate of a complex number"), // to translate in deutsch
		OUString::createFromAscii("Returns the conjugate of a complex number"),
		OUString::createFromAscii("Returns the conjugate of a complex number"),
		OUString::createFromAscii("Returns the conjugate of a complex number"),
		OUString::createFromAscii("Returns the conjugate of a complex number"),
		OUString::createFromAscii("Returns the conjugate of a complex number"),
		OUString::createFromAscii("Returns the conjugate of a complex number")
		},
		{
		OUString::createFromAscii("arg"),
		OUString::createFromAscii("Renvoie un argument d'un nombre complexe"),
		OUString::createFromAscii("Returns an argument of a complex number"),
		OUString::createFromAscii("Returns an argument of a complex number"), // to translate in deutsch
		OUString::createFromAscii("Returns an argument of a complex number"),
		OUString::createFromAscii("Returns an argument of a complex number"),
		OUString::createFromAscii("Returns an argument of a complex number"),
		OUString::createFromAscii("Returns an argument of a complex number"),
		OUString::createFromAscii("Returns an argument of a complex number"),
		OUString::createFromAscii("Returns an argument of a complex number")
		},
		{
		OUString::createFromAscii("evalc"),
		OUString::createFromAscii("Renvoie une expression complexe simplifi")+eAigu+OUString::createFromAscii("e sous la forme real+i*imag"),
		OUString::createFromAscii("Returns an complex expression simplified with the format real+i*imag"),
		OUString::createFromAscii("Returns an complex expression simplified with the format real+i*imag"), // to translate in deutsch
		OUString::createFromAscii("Returns an complex expression simplified with the format real+i*imag"),
		OUString::createFromAscii("Returns an complex expression simplified with the format real+i*imag"),
		OUString::createFromAscii("Returns an complex expression simplified with the format real+i*imag"),
		OUString::createFromAscii("Returns an complex expression simplified with the format real+i*imag"),
		OUString::createFromAscii("Returns an complex expression simplified with the format real+i*imag"),
		OUString::createFromAscii("Returns an complex expression simplified with the format real+i*imag")
		},
		{
		OUString::createFromAscii("subst"),
		OUString::createFromAscii("Remplace une variable par sa valeur dans une expression"),
		OUString::createFromAscii("Substitutes a value for a variable in an expression"),
		OUString::createFromAscii("Substitutes a value for a variable in an expression"), // to translate in deutsch
		OUString::createFromAscii("Substitutes a value for a variable in an expression"),
		OUString::createFromAscii("Substitutes a value for a variable in an expression"),
		OUString::createFromAscii("Substitutes a value for a variable in an expression"),
		OUString::createFromAscii("Substitutes a value for a variable in an expression"),
		OUString::createFromAscii("Substitutes a value for a variable in an expression"),
		OUString::createFromAscii("Substitutes a value for a variable in an expression")
		},
		{
		OUString::createFromAscii("evalf"),
		EAigu+OUString::createFromAscii("value num")+eAigu+OUString::createFromAscii("riquement une expression math")+eAigu+OUString::createFromAscii("matique"),
		OUString::createFromAscii("Numerical evaluation of a mathematical expression"),
		OUString::createFromAscii("Numerical evaluation of a mathematical expression"), // to translate in deutsch
		OUString::createFromAscii("Numerical evaluation of a mathematical expression"),
		OUString::createFromAscii("Numerical evaluation of a mathematical expression"),
		OUString::createFromAscii("Numerical evaluation of a mathematical expression"),
		OUString::createFromAscii("Numerical evaluation of a mathematical expression"),
		OUString::createFromAscii("Numerical evaluation of a mathematical expression"),
		OUString::createFromAscii("Numerical evaluation of a mathematical expression")
		},
		{
		OUString::createFromAscii("cexact"),
		OUString::createFromAscii("Convertit l'expression en une expression rationnelle ou r")+eAigu+OUString::createFromAscii("elle"),
		OUString::createFromAscii("Converts the expression to a rational or real expression"),
		OUString::createFromAscii("Converts the expression to a rational or real expression"), // to translate in deutsch
		OUString::createFromAscii("Converts the expression to a rational or real expression"),
		OUString::createFromAscii("Converts the expression to a rational or real expression"),
		OUString::createFromAscii("Converts the expression to a rational or real expression"),
		OUString::createFromAscii("Converts the expression to a rational or real expression"),
		OUString::createFromAscii("Converts the expression to a rational or real expression"),
		OUString::createFromAscii("Converts the expression to a rational or real expression")
		},
		{
		OUString::createFromAscii("idivis"),
		OUString::createFromAscii("Renvoie la liste des diviseurs d'un entier"),
		OUString::createFromAscii("Returns the list of divisors of an integer"),
		OUString::createFromAscii("Returns the list of divisors of an integer"), // to translate in deutsch
		OUString::createFromAscii("Returns the list of divisors of an integer"),
		OUString::createFromAscii("Returns the list of divisors of an integer"),
		OUString::createFromAscii("Returns the list of divisors of an integer"),
		OUString::createFromAscii("Returns the list of divisors of an integer"),
		OUString::createFromAscii("Returns the list of divisors of an integer"),
		OUString::createFromAscii("Returns the list of divisors of an integer")
		},
		{
		OUString::createFromAscii("deriver"),
		OUString::createFromAscii("D")+eAigu+OUString::createFromAscii("riv")+eAigu+OUString::createFromAscii("e par rapport au second argument(facultatif)"),
		OUString::createFromAscii("Returns the derivative"),
		OUString::createFromAscii("Returns the derivative"), // to translate in deutsch
		OUString::createFromAscii("Returns the derivative"),
		OUString::createFromAscii("Returns the derivative"),
		OUString::createFromAscii("Returns the derivative"),
		OUString::createFromAscii("Returns the derivative"),
		OUString::createFromAscii("Returns the derivative"),
		OUString::createFromAscii("Returns the derivative")
		},
		{
		OUString::createFromAscii("integrer"),
		OUString::createFromAscii("Primitive (la variable d'int")+eAigu+OUString::createFromAscii("gration est le 2i")+eGrave+OUString::createFromAscii("me argument) ou valeur de l'int")+eAigu+OUString::createFromAscii("grale (bornes a et b)"),
		OUString::createFromAscii("Returns the integral"),
		OUString::createFromAscii("Returns the integral"), // to translate in deutsch
		OUString::createFromAscii("Returns the integral"),
		OUString::createFromAscii("Returns the integral"),
		OUString::createFromAscii("Returns the integral"),
		OUString::createFromAscii("Returns the integral"),
		OUString::createFromAscii("Returns the integral"),
		OUString::createFromAscii("Returns the integral")
		},
		{
		OUString::createFromAscii("csomme"),
		OUString::createFromAscii("Somme des valeurs de l'expression quand la variable va de a ")+aGrave+OUString::createFromAscii(" b"),
		OUString::createFromAscii("Discret sum when the variable goes from a to b"),
		OUString::createFromAscii("Discret sum when the variable goes from a to b"), // to translate in deutsch
		OUString::createFromAscii("Discret sum when the variable goes from a to b"),
		OUString::createFromAscii("Discret sum when the variable goes from a to b"),
		OUString::createFromAscii("Discret sum when the variable goes from a to b"),
		OUString::createFromAscii("Discret sum when the variable goes from a to b"),
		OUString::createFromAscii("Discret sum when the variable goes from a to b"),
		OUString::createFromAscii("Discret sum when the variable goes from a to b")
		},
		{
		OUString::createFromAscii("cproduit"),
		OUString::createFromAscii("Produit des valeurs de l'expression quand la variable va de a ")+aGrave+OUString::createFromAscii(" b"),
		OUString::createFromAscii("Discret product when the variable goes from a to b"),
		OUString::createFromAscii("Discret product when the variable goes from a to b"), // to translate in deutsch
		OUString::createFromAscii("Discret product when the variable goes from a to b"),
		OUString::createFromAscii("Discret product when the variable goes from a to b"),
		OUString::createFromAscii("Discret product when the variable goes from a to b"),
		OUString::createFromAscii("Discret product when the variable goes from a to b"),
		OUString::createFromAscii("Discret product when the variable goes from a to b"),
		OUString::createFromAscii("Discret product when the variable goes from a to b")
		},
		{
		OUString::createFromAscii("limite"),
		OUString::createFromAscii("Limite d'une expression en un point (a ou +/-infinity)"),
		OUString::createFromAscii("Limit of an expression at a limit point (a or +/-infinity)"),
		OUString::createFromAscii("Limit of an expression at a limit point (a or +/-infinity)"), // to translate in deutsch
		OUString::createFromAscii("Limit of an expression at a limit point (a or +/-infinity)"),
		OUString::createFromAscii("Limit of an expression at a limit point (a or +/-infinity)"),
		OUString::createFromAscii("Limit of an expression at a limit point (a or +/-infinity)"),
		OUString::createFromAscii("Limit of an expression at a limit point (a or +/-infinity)"),
		OUString::createFromAscii("Limit of an expression at a limit point (a or +/-infinity)"),
		OUString::createFromAscii("Limit of an expression at a limit point (a or +/-infinity)")
		},
		{
		OUString::createFromAscii("makelist"),
		OUString::createFromAscii("Renvoie un tableau de valeurs fait ")+aGrave+OUString::createFromAscii(" partir d'une fonction"),
		OUString::createFromAscii("Returns a list made with a function"),
		OUString::createFromAscii("Returns a list made with a function"), // to translate in deutsch
		OUString::createFromAscii("Returns a list made with a function"),
		OUString::createFromAscii("Returns a list made with a function"),
		OUString::createFromAscii("Returns a list made with a function"),
		OUString::createFromAscii("Returns a list made with a function"),
		OUString::createFromAscii("Returns a list made with a function"),
		OUString::createFromAscii("Returns a list made with a function")
		},
		{
		OUString::createFromAscii("czeros"),
		OUString::createFromAscii("Renvoie la liste des z")+eAigu+OUString::createFromAscii("ros complexes de l'expression"),
		OUString::createFromAscii("Returns the complex zeros of the expression"),
		OUString::createFromAscii("Returns the complex zeros of the expression"), // to translate in deutsch
		OUString::createFromAscii("Returns the complex zeros of the expression"),
		OUString::createFromAscii("Returns the complex zeros of the expression"),
		OUString::createFromAscii("Returns the complex zeros of the expression"),
		OUString::createFromAscii("Returns the complex zeros of the expression"),
		OUString::createFromAscii("Returns the complex zeros of the expression"),
		OUString::createFromAscii("Returns the complex zeros of the expression")
		},
		{
		OUString::createFromAscii("zeros"),
		OUString::createFromAscii("Renvoie la liste des z")+eAigu+OUString::createFromAscii("ros r")+eAigu+OUString::createFromAscii("els ou complexes (selon le mode) de l'expression"),
		OUString::createFromAscii("Returns the zeros (reals or complex according to the mode) of the expression"),
		OUString::createFromAscii("Returns the zeros (reals or complex according to the mode) of the expression"), // to translate in deutsch
		OUString::createFromAscii("Returns the zeros (reals or complex according to the mode) of the expression"),
		OUString::createFromAscii("Returns the zeros (reals or complex according to the mode) of the expression"),
		OUString::createFromAscii("Returns the zeros (reals or complex according to the mode) of the expression"),
		OUString::createFromAscii("Returns the zeros (reals or complex according to the mode) of the expression"),
		OUString::createFromAscii("Returns the zeros (reals or complex according to the mode) of the expression"),
		OUString::createFromAscii("Returns the zeros (reals or complex according to the mode) of the expression")
		},
		{
		OUString::createFromAscii("factorial"),
		OUString::createFromAscii("Renvoie n!"),
		OUString::createFromAscii("Returns n!"),
		OUString::createFromAscii("Returns n!"), // to translate in deutsch
		OUString::createFromAscii("Returns n!"),
		OUString::createFromAscii("Returns n!"),
		OUString::createFromAscii("Returns n!"),
		OUString::createFromAscii("Returns n!"),
		OUString::createFromAscii("Returns n!"),
		OUString::createFromAscii("Returns n!")
		},
		{
		OUString::createFromAscii("factoriser"),
		OUString::createFromAscii("Factorisation d'un polyn")+oCirconflexe+OUString::createFromAscii("me"),
		OUString::createFromAscii("Factorizes a polynomial"),
		OUString::createFromAscii("Factorizes a polynomial"), // to translate in deutsch
		OUString::createFromAscii("Factorizes a polynomial"),
		OUString::createFromAscii("Factorizes a polynomial"),
		OUString::createFromAscii("Factorizes a polynomial"),
		OUString::createFromAscii("Factorizes a polynomial"),
		OUString::createFromAscii("Factorizes a polynomial"),
		OUString::createFromAscii("Factorizes a polynomial")
		},
		{
		OUString::createFromAscii("factoriserSurC"),
		OUString::createFromAscii("Factorisation de l'expression dans C"),
		OUString::createFromAscii("Factorisation of the expression in C"),
		OUString::createFromAscii("Factorisation of the expression in C"), // to translate in deutsch
		OUString::createFromAscii("Factorisation of the expression in C"),
		OUString::createFromAscii("Factorisation of the expression in C"),
		OUString::createFromAscii("Factorisation of the expression in C"),
		OUString::createFromAscii("Factorisation of the expression in C"),
		OUString::createFromAscii("Factorisation of the expression in C"),
		OUString::createFromAscii("Factorisation of the expression in C")
		},
		{
		OUString::createFromAscii("developper"),
		OUString::createFromAscii("Distribue totalement * et / par rapport ")+aGrave+OUString::createFromAscii(" + et -"),
		OUString::createFromAscii("Full distribution of * and / over + and -"),
		OUString::createFromAscii("Full distribution of * and / over + and -"), // to translate in deutsch
		OUString::createFromAscii("Full distribution of * and / over + and -"),
		OUString::createFromAscii("Full distribution of * and / over + and -"),
		OUString::createFromAscii("Full distribution of * and / over + and -"),
		OUString::createFromAscii("Full distribution of * and / over + and -"),
		OUString::createFromAscii("Full distribution of * and / over + and -"),
		OUString::createFromAscii("Full distribution of * and / over + and -")
		},
		{
		OUString::createFromAscii("iquo"),
		OUString::createFromAscii("Quotient euclidien de 2 entiers"),
		OUString::createFromAscii("Euclidean quotient of 2 integers"),
		OUString::createFromAscii("Euclidean quotient of 2 integers"), // to translate in deutsch
		OUString::createFromAscii("Euclidean quotient of 2 integers"),
		OUString::createFromAscii("Euclidean quotient of 2 integers"),
		OUString::createFromAscii("Euclidean quotient of 2 integers"),
		OUString::createFromAscii("Euclidean quotient of 2 integers"),
		OUString::createFromAscii("Euclidean quotient of 2 integers"),
		OUString::createFromAscii("Euclidean quotient of 2 integers")
		},
		{
		OUString::createFromAscii("irem"),
		OUString::createFromAscii("Reste euclidien de 2 entiers"),
		OUString::createFromAscii("Euclidean remainder of 2 integers"),
		OUString::createFromAscii("Euclidean remainder of 2 integers"), // to translate in deutsch
		OUString::createFromAscii("Euclidean remainder of 2 integers"),
		OUString::createFromAscii("Euclidean remainder of 2 integers"),
		OUString::createFromAscii("Euclidean remainder of 2 integers"),
		OUString::createFromAscii("Euclidean remainder of 2 integers"),
		OUString::createFromAscii("Euclidean remainder of 2 integers"),
		OUString::createFromAscii("Euclidean remainder of 2 integers")
		},
		{
		OUString::createFromAscii("quo"),
		OUString::createFromAscii("Quotient euclidien de 2 polyn")+oCirconflexe+OUString::createFromAscii("mes"),
		OUString::createFromAscii("Euclidean quotient of 2 polynomials"),
		OUString::createFromAscii("Euclidean quotient of 2 polynomials"), // to translate in deutsch
		OUString::createFromAscii("Euclidean quotient of 2 polynomials"),
		OUString::createFromAscii("Euclidean quotient of 2 polynomials"),
		OUString::createFromAscii("Euclidean quotient of 2 polynomials"),
		OUString::createFromAscii("Euclidean quotient of 2 polynomials"),
		OUString::createFromAscii("Euclidean quotient of 2 polynomials"),
		OUString::createFromAscii("Euclidean quotient of 2 polynomials")
		},
		{
		OUString::createFromAscii("rem"),
		OUString::createFromAscii("Reste euclidien de 2 polyn")+oCirconflexe+OUString::createFromAscii("mes"),
		OUString::createFromAscii("Euclidean remainder of 2 polynomials"),
		OUString::createFromAscii("Euclidean remainder of 2 polynomials"), // to translate in deutsch
		OUString::createFromAscii("Euclidean remainder of 2 polynomials"),
		OUString::createFromAscii("Euclidean remainder of 2 polynomials"),
		OUString::createFromAscii("Euclidean remainder of 2 polynomials"),
		OUString::createFromAscii("Euclidean remainder of 2 polynomials"),
		OUString::createFromAscii("Euclidean remainder of 2 polynomials"),
		OUString::createFromAscii("Euclidean remainder of 2 polynomials")
		},
		{
		OUString::createFromAscii("resoudre"),
		OUString::createFromAscii("R")+eAigu+OUString::createFromAscii("sout une ")+eAigu+OUString::createFromAscii("quation"),
		OUString::createFromAscii("Solves a polynomial equation"),
		OUString::createFromAscii("Solves a polynomial equation"), // to translate in deutsch
		OUString::createFromAscii("Solves a polynomial equation"),
		OUString::createFromAscii("Solves a polynomial equation"),
		OUString::createFromAscii("Solves a polynomial equation"),
		OUString::createFromAscii("Solves a polynomial equation"),
		OUString::createFromAscii("Solves a polynomial equation"),
		OUString::createFromAscii("Solves a polynomial equation")
		},
		{
		OUString::createFromAscii("resoudreDansC"),
		OUString::createFromAscii("R")+eAigu+OUString::createFromAscii("sout une ")+eAigu+OUString::createFromAscii("quation dans C"),
		OUString::createFromAscii("Solves a polynomial equation in C"),
		OUString::createFromAscii("Solves a polynomial equation in C"), // to translate in deutsch
		OUString::createFromAscii("Solves a polynomial equation in C"),
		OUString::createFromAscii("Solves a polynomial equation in C"),
		OUString::createFromAscii("Solves a polynomial equation in C"),
		OUString::createFromAscii("Solves a polynomial equation in C"),
		OUString::createFromAscii("Solves a polynomial equation in C"),
		OUString::createFromAscii("Solves a polynomial equation in C")
		},
		{
		OUString::createFromAscii("resoudreNumerique"),
		OUString::createFromAscii("R")+eAigu+OUString::createFromAscii("sout numériquement une ")+eAigu+OUString::createFromAscii("quation"),
		OUString::createFromAscii("Numerical solution of an equation"),
		OUString::createFromAscii("Numerical solution of an equation"), // to translate in deutsch
		OUString::createFromAscii("Numerical solution of an equation"),
		OUString::createFromAscii("Numerical solution of an equation"),
		OUString::createFromAscii("Numerical solution of an equation"),
		OUString::createFromAscii("Numerical solution of an equation"),
		OUString::createFromAscii("Numerical solution of an equation"),
		OUString::createFromAscii("Numerical solution of an equation")
		},
		{
		OUString::createFromAscii("gcd"),
		OUString::createFromAscii("PGCD de 2 polyn")+oCirconflexe+OUString::createFromAscii("mes de plusieurs variables ou de 2 entiers ou de 2 rationnels"),
		OUString::createFromAscii("Returns the greatest common divisor of 2 polynomials of several variables or of 2 integers or of 2 rationnals"),
		OUString::createFromAscii("Returns the greatest common divisor of 2 polynomials of several variables or of 2 integers or of 2 rationnals"), // to translate in deutsch
		
		OUString::createFromAscii("Returns the greatest common divisor of 2 polynomials of several variables or of 2 integers or of 2 rationnals"),
		OUString::createFromAscii("Returns the greatest common divisor of 2 polynomials of several variables or of 2 integers or of 2 rationnals"),
		OUString::createFromAscii("Returns the greatest common divisor of 2 polynomials of several variables or of 2 integers or of 2 rationnals"),
		OUString::createFromAscii("Returns the greatest common divisor of 2 polynomials of several variables or of 2 integers or of 2 rationnals"),
		OUString::createFromAscii("Returns the greatest common divisor of 2 polynomials of several variables or of 2 integers or of 2 rationnals"),
		OUString::createFromAscii("Returns the greatest common divisor of 2 polynomials of several variables or of 2 integers or of 2 rationnals")
		},
		{
		OUString::createFromAscii("lcm"),
		OUString::createFromAscii("PPCM de 2 polyn")+oCirconflexe+OUString::createFromAscii("mes de plusieurs variables ou de 2 entiers ou de 2 rationnels"),
		OUString::createFromAscii("Returns the lowest common multiple of 2 polynomials of several variables or of 2 integers or of 2 rationnals"),
		OUString::createFromAscii("Returns the lowest common multiple of 2 polynomials of several variables or of 2 integers or of 2 rationnals"), // to translate in deutsch
		
		OUString::createFromAscii("Returns the lowest common multiple of 2 polynomials of several variables or of 2 integers or of 2 rationnals"),
		OUString::createFromAscii("Returns the lowest common multiple of 2 polynomials of several variables or of 2 integers or of 2 rationnals"),
		OUString::createFromAscii("Returns the lowest common multiple of 2 polynomials of several variables or of 2 integers or of 2 rationnals"),
		OUString::createFromAscii("Returns the lowest common multiple of 2 polynomials of several variables or of 2 integers or of 2 rationnals"),
		OUString::createFromAscii("Returns the lowest common multiple of 2 polynomials of several variables or of 2 integers or of 2 rationnals"),
		OUString::createFromAscii("Returns the lowest common multiple of 2 polynomials of several variables or of 2 integers or of 2 rationnals")
		},
		{
		OUString::createFromAscii("formeCanonique"),
		OUString::createFromAscii("Forme canonique du trin")+oCirconflexe+OUString::createFromAscii("me de degr")+eAigu+OUString::createFromAscii(" 2"),
		OUString::createFromAscii("Canonical form of a 2 degree polynom"),
		OUString::createFromAscii("Canonical form of a 2 degree polynom"), // to translate in deutsch
		OUString::createFromAscii("Canonical form of a 2 degree polynom"),
		OUString::createFromAscii("Canonical form of a 2 degree polynom"),
		OUString::createFromAscii("Canonical form of a 2 degree polynom"),
		OUString::createFromAscii("Canonical form of a 2 degree polynom"),
		OUString::createFromAscii("Canonical form of a 2 degree polynom"),
		OUString::createFromAscii("Canonical form of a 2 degree polynom")
		},
		{
		OUString::createFromAscii("factoriserEntier"),
		OUString::createFromAscii("Factorisation d'un entier en facteurs premiers"),
		OUString::createFromAscii("Factorization of an integer into prime factors"),
		OUString::createFromAscii("Factorization of an integer into prime factors"), // to translate in deutsch
		OUString::createFromAscii("Factorization of an integer into prime factors"),
		OUString::createFromAscii("Factorization of an integer into prime factors"),
		OUString::createFromAscii("Factorization of an integer into prime factors"),
		OUString::createFromAscii("Factorization of an integer into prime factors"),
		OUString::createFromAscii("Factorization of an integer into prime factors"),
		OUString::createFromAscii("Factorization of an integer into prime factors")
		},
		{
		OUString::createFromAscii("isprime"),
		OUString::createFromAscii("Test de primalit")+eAigu+OUString::createFromAscii(" (=true ou false)"),
		OUString::createFromAscii("Probabilistic primality testing (=true or false)"),
		OUString::createFromAscii("Probabilistic primality testing (=true or false)"), // to translate in deutsch
		OUString::createFromAscii("Probabilistic primality testing (=true or false)"),
		OUString::createFromAscii("Probabilistic primality testing (=true or false)"),
		OUString::createFromAscii("Probabilistic primality testing (=true or false)"),
		OUString::createFromAscii("Probabilistic primality testing (=true or false)"),
		OUString::createFromAscii("Probabilistic primality testing (=true or false)"),
		OUString::createFromAscii("Probabilistic primality testing (=true or false)")
		},
		{
		OUString::createFromAscii("carcsin"),
		OUString::createFromAscii("Renvoie l'arcsinus d'une expression math")+eAigu+OUString::createFromAscii("matique"),
		OUString::createFromAscii("Arcsinus"),
		OUString::createFromAscii("Return Arcsinus"),
		OUString::createFromAscii("Return Arcsinus"), // to translate in deutsch
		OUString::createFromAscii("Arcsinus"),
		OUString::createFromAscii("Arcsinus"),
		OUString::createFromAscii("Arcsinus"),
		OUString::createFromAscii("Arcsinus"),
		OUString::createFromAscii("Arcsinus")
		},
		{
		OUString::createFromAscii("carccos"),
		OUString::createFromAscii("Renvoie l'arccosinus d'une expression math")+eAigu+OUString::createFromAscii("matique"),
		OUString::createFromAscii("Return Arccos"),
		OUString::createFromAscii("Return Arccos"), // to translate in deutsch
		OUString::createFromAscii("Arccos"),
		OUString::createFromAscii("Arccos"),
		OUString::createFromAscii("Arccos"),
		OUString::createFromAscii("Arccos"),
		OUString::createFromAscii("Arccos"),
		OUString::createFromAscii("Arccos")
		},
		{
		OUString::createFromAscii("carctan"),
		OUString::createFromAscii("Renvoie l'arctangente d'une expression math")+eAigu+OUString::createFromAscii("matique"),
		OUString::createFromAscii("Return Arctan"),
		OUString::createFromAscii("Return Arctan"), // to translate in deutsch
		OUString::createFromAscii("Arctan"),
		OUString::createFromAscii("Arctan"),
		OUString::createFromAscii("Arctan"),
		OUString::createFromAscii("Arctan"),
		OUString::createFromAscii("Arctan"),
		OUString::createFromAscii("Arctan")
		},
		{
		OUString::createFromAscii("divis"),
		OUString::createFromAscii("Renvoie la liste des diviseurs d'un polyn")+oCirconflexe+OUString::createFromAscii("me"),
		OUString::createFromAscii("Returns the list of divisors of a polynomial"),
		OUString::createFromAscii("Returns the list of divisors of a polynomial"), // to translate in deutsch
		OUString::createFromAscii("Returns the list of divisors of a polynomial"),
		OUString::createFromAscii("Returns the list of divisors of a polynomial"),
		OUString::createFromAscii("Returns the list of divisors of a polynomial"),
		OUString::createFromAscii("Returns the list of divisors of a polynomial"),
		OUString::createFromAscii("Returns the list of divisors of a polynomial"),
		OUString::createFromAscii("Returns the list of divisors of a polynomial")
		},
		{
		OUString::createFromAscii("taylor"),
		OUString::createFromAscii("D")+eAigu+OUString::createFromAscii("veloppement limit")+eAigu+OUString::createFromAscii(" ou asymptotique (par d")+eAigu+OUString::createFromAscii("faut x=0 et ordre relatif=5)"),
		OUString::createFromAscii("Series expansion at finite or infinite points (by default x=0 and order=5)"),
		OUString::createFromAscii("Series expansion at finite or infinite points (by default x=0 and order=5)"), // to translate in deutsch
		OUString::createFromAscii("Series expansion at finite or infinite points (by default x=0 and order=5)"),
		OUString::createFromAscii("Series expansion at finite or infinite points (by default x=0 and order=5)"),
		OUString::createFromAscii("Series expansion at finite or infinite points (by default x=0 and order=5)"),
		OUString::createFromAscii("Series expansion at finite or infinite points (by default x=0 and order=5)"),
		OUString::createFromAscii("Series expansion at finite or infinite points (by default x=0 and order=5)"),
		OUString::createFromAscii("Series expansion at finite or infinite points (by default x=0 and order=5)")
		},
		{
		OUString::createFromAscii("prevprime"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("nextprime"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("isPseudoPrime"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("euler"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("latex"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("mathml"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("developperTranscendant"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("ccosh"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("csinh"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("ctanh"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("ccotan"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("carccosh"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("carcsinh"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("carctanh"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("carccotan"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("iegcd"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("iabcuv"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("egcd"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("abcuv"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("fmax"),
		OUString::createFromAscii("Renvoie l'abscisse d'un maximum de l'expression"),
		OUString::createFromAscii("Returns the abscissa of a maximum of the expression"),
		OUString::createFromAscii("Returns the abscissa of a maximum of the expression"), // to translate in deutsch
		OUString::createFromAscii("Returns the abscissa of a maximum of the expression"),
		OUString::createFromAscii("Returns the abscissa of a maximum of the expression"),
		OUString::createFromAscii("Returns the abscissa of a maximum of the expression"),
		OUString::createFromAscii("Returns the abscissa of a maximum of the expression"),
		OUString::createFromAscii("Returns the abscissa of a maximum of the expression"),
		OUString::createFromAscii("Returns the abscissa of a maximum of the expression")
		},
		{
		OUString::createFromAscii("fmin"),
		OUString::createFromAscii("Renvoie l'abscisse d'un minimum de l'expression"),
		OUString::createFromAscii("Returns the abscissa of a minimum of the expression"),
		OUString::createFromAscii("Returns the abscissa of a minimum of the expression"), // to translate in deutsch
		OUString::createFromAscii("Returns the abscissa of a minimum of the expression"),
		OUString::createFromAscii("Returns the abscissa of a minimum of the expression"),
		OUString::createFromAscii("Returns the abscissa of a minimum of the expression"),
		OUString::createFromAscii("Returns the abscissa of a minimum of the expression"),
		OUString::createFromAscii("Returns the abscissa of a minimum of the expression"),
		OUString::createFromAscii("Returns the abscissa of a minimum of the expression")
		},
		{
		OUString::createFromAscii("dsolve"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("linsolve"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("egv"),
		OUString::createFromAscii("Calcule les vecteurs propres d'une matrice diagonalisable"),
		OUString::createFromAscii("Computes the eigenvectors of a diagonalizable matrix"),
		OUString::createFromAscii("Computes the eigenvectors of a diagonalizable matrix"), // to translate in deutsch
		OUString::createFromAscii("Computes the eigenvectors of a diagonalizable matrix"),
		OUString::createFromAscii("Computes the eigenvectors of a diagonalizable matrix"),
		OUString::createFromAscii("Computes the eigenvectors of a diagonalizable matrix"),
		OUString::createFromAscii("Computes the eigenvectors of a diagonalizable matrix"),
		OUString::createFromAscii("Computes the eigenvectors of a diagonalizable matrix"),
		OUString::createFromAscii("Computes the eigenvectors of a diagonalizable matrix")
		},
		{
		OUString::createFromAscii("eigenvals"),
		OUString::createFromAscii("Renvoie la liste des valeurs propres (calculables) d'une matrice"),
		OUString::createFromAscii("Returns the list of the (calculable) eigenvalues of a matrix"),
		OUString::createFromAscii("Returns the list of the (calculable) eigenvalues of a matrix"), // to translate in deutsch
		OUString::createFromAscii("Returns the list of the (calculable) eigenvalues of a matrix"),
		OUString::createFromAscii("Returns the list of the (calculable) eigenvalues of a matrix"),
		OUString::createFromAscii("Returns the list of the (calculable) eigenvalues of a matrix"),
		OUString::createFromAscii("Returns the list of the (calculable) eigenvalues of a matrix"),
		OUString::createFromAscii("Returns the list of the (calculable) eigenvalues of a matrix"),
		OUString::createFromAscii("Returns the list of the (calculable) eigenvalues of a matrix")
		},
		{
		OUString::createFromAscii("matinv"),
		OUString::createFromAscii("Calcul l'inverse d'une matrice"),
		OUString::createFromAscii("Inverse of a matrix"),
		OUString::createFromAscii("Inverse of a matrix"), // to translate in deutsch
		OUString::createFromAscii("Inverse of a matrix"),
		OUString::createFromAscii("Inverse of a matrix"),
		OUString::createFromAscii("Inverse of a matrix"),
		OUString::createFromAscii("Inverse of a matrix"),
		OUString::createFromAscii("Inverse of a matrix"),
		OUString::createFromAscii("Inverse of a matrix")
		},
		{
		OUString::createFromAscii("matadd"),
		OUString::createFromAscii("Additionne 2 matrices"),
		OUString::createFromAscii("Matrix sum"),
		OUString::createFromAscii("Matrix sum"), // to translate in deutsch
		OUString::createFromAscii("Matrix sum"),
		OUString::createFromAscii("Matrix sum"),
		OUString::createFromAscii("Matrix sum"),
		OUString::createFromAscii("Matrix sum"),
		OUString::createFromAscii("Matrix sum"),
		OUString::createFromAscii("Matrix sum")
		},
		{
		OUString::createFromAscii("matsub"),
		OUString::createFromAscii("Soustrait 2 matrices"),
		OUString::createFromAscii("Matrix subtraction"),
		OUString::createFromAscii("Matrix subtraction"), // to translate in deutsch
		OUString::createFromAscii("Matrix subtraction"),
		OUString::createFromAscii("Matrix subtraction"),
		OUString::createFromAscii("Matrix subtraction"),
		OUString::createFromAscii("Matrix subtraction"),
		OUString::createFromAscii("Matrix subtraction"),
		OUString::createFromAscii("Matrix subtraction")
		},
		{
		OUString::createFromAscii("matmult"),
		OUString::createFromAscii("Multiplication de 2 matrices")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("Matrix product"),
		OUString::createFromAscii("Matrix product"), // to translate in deutsch
		OUString::createFromAscii("Matrix product"),
		OUString::createFromAscii("Matrix product"),
		OUString::createFromAscii("Matrix product"),
		OUString::createFromAscii("Matrix product"),
		OUString::createFromAscii("Matrix product"),
		OUString::createFromAscii("Matrix product")
		},
		{
		OUString::createFromAscii("matpow"),
		EAigu+OUString::createFromAscii("l")+eGrave+OUString::createFromAscii("ve une matrice ")+aGrave+OUString::createFromAscii(" une puissance"),
		OUString::createFromAscii("Calculates the power of a matrix"),
		OUString::createFromAscii("Calculates the power of a matrix"), // to translate in deutsch
		OUString::createFromAscii("Calculates the power of a matrix"),
		OUString::createFromAscii("Calculates the power of a matrix"),
		OUString::createFromAscii("Calculates the power of a matrix"),
		OUString::createFromAscii("Calculates the power of a matrix"),
		OUString::createFromAscii("Calculates the power of a matrix"),
		OUString::createFromAscii("Calculates the power of a matrix")
		},
		{
		OUString::createFromAscii("det"),
		OUString::createFromAscii("D")+eAigu+OUString::createFromAscii("terminant d'une matrice carr")+eAigu+OUString::createFromAscii("e"),
		OUString::createFromAscii("Determinant of a square matrix"),
		OUString::createFromAscii("Determinant of a square matrix"), // to translate in deutsch
		OUString::createFromAscii("Determinant of a square matrix"),
		OUString::createFromAscii("Determinant of a square matrix"),
		OUString::createFromAscii("Determinant of a square matrix"),
		OUString::createFromAscii("Determinant of a square matrix"),
		OUString::createFromAscii("Determinant of a square matrix"),
		OUString::createFromAscii("Determinant of a square matrix")
		},
		{
		OUString::createFromAscii("cas1"),
		OUString::createFromAscii("usage interne"),
		OUString::createFromAscii("internal use"),
		OUString::createFromAscii("internal use"), // to translate in deutsch
		OUString::createFromAscii("internal use"),
		OUString::createFromAscii("internal use"),
		OUString::createFromAscii("internal use"),
		OUString::createFromAscii("internal use"),
		OUString::createFromAscii("internal use"),
		OUString::createFromAscii("internal use")
		},
		{
		OUString::createFromAscii("cas2"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("cas3"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("cas4"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},
		{
		OUString::createFromAscii("cas5"),
		OUString::createFromAscii("non encore impl")+eAigu+OUString::createFromAscii("ment")+eAigu,
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("nicht implementiert"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented"),
		OUString::createFromAscii("not yet implemented")
		},

		{ // garde fou. Normalement, on ne renvoie jamais cet élément.
		OUString::createFromAscii("internalError"),
		OUString::createFromAscii("erreurInterne"),
		OUString::createFromAscii("internalError"),
		OUString::createFromAscii("internalError"), // to translate in deutsch
		OUString::createFromAscii("internalError"),
		OUString::createFromAscii("internalError"),
		OUString::createFromAscii("internalError"),
		OUString::createFromAscii("internalError"),
		OUString::createFromAscii("internalError"),
		OUString::createFromAscii("internalError")
		} };
	sal_Int32 i=0;
	sal_Int32 lang=0;
	if (aFuncLoc.Language.equalsAscii("fr"))
	{
		lang=1;
	}
        else if(aFuncLoc.Language.equalsAscii("de"))
        {
		lang=3;                
        }
	else
	{	
		// default is english
		lang=2;
	}
	while (i< nNumFunctions && !aProgrammaticName.equals(FunctionDescription[i][0])){
		i++;
	}
	return FunctionDescription[i][lang];
}
 
OUString SAL_CALL CmathOOoCASAddinImpl::getDisplayArgumentName( const OUString& aProgrammaticName, sal_Int32 nArgument )
{   
    OUString eAigu((sal_Unicode)0x00E9);
    OUString EAigu((sal_Unicode)0x00C9);
    OUString aGrave((sal_Unicode)0x00E0);
    OUString eGrave((sal_Unicode)0x00E8);
    OUString oCirconflexe((sal_Unicode)0x00F4);
    OUString oTrema((sal_Unicode)0x00F6);
    OUString uTrema((sal_Unicode)0x00FC);
    OUString aTrema((sal_Unicode)0x00E4);

    // if you want to add your own language, send me your translation at : cdeval@free.fr
    sal_Int32 lang=0;
    OUString expression;
    OUString entier;
    OUString notYetImplemented;
    OUString matrice;
    OUString vecteur;
    if (aFuncLoc.Language.equalsAscii("fr"))
    {
       lang=1; // french
       expression=OUString::createFromAscii("une expression");
       entier=OUString::createFromAscii("un entier");
       notYetImplemented=OUString::createFromAscii("fonction non disponible");
	   matrice=OUString::createFromAscii("une matrice");
	   vecteur=OUString::createFromAscii("un vecteur");
    }
    else if (aFuncLoc.Language.equalsAscii("de"))
    {
       lang=3; // deutsch
       expression=OUString::createFromAscii("ein Ausdruck");
       entier=OUString::createFromAscii("eine natürliche Zahl");
       notYetImplemented=OUString::createFromAscii("noch nicht verfügbar");
	   matrice=OUString::createFromAscii("eine matrix");
	   vecteur=OUString::createFromAscii("ein vektor");
    }
    else
    {	
       lang=2; // english
       expression=OUString::createFromAscii("an expression");
       entier=OUString::createFromAscii("an integer");
       notYetImplemented=OUString::createFromAscii("not yet available");
	   matrice=OUString::createFromAscii("a matrix");
	   vecteur=OUString::createFromAscii("a vector");
    }

    // un seul paramètre
    if (aProgrammaticName.equalsAscii("cas"))
	return expression;
    if (aProgrammaticName.equalsAscii("casCompil"))
	return expression;
    if (aProgrammaticName.equalsAscii("casFonction"))
	return expression;
    if (aProgrammaticName.equalsAscii("simplifier"))
	return expression;
    if (aProgrammaticName.equalsAscii("cadd"))
	return expression;
    if (aProgrammaticName.equalsAscii("csub"))
	return expression;
    if (aProgrammaticName.equalsAscii("cmul"))
	return expression;
    if (aProgrammaticName.equalsAscii("cdiv"))
	return expression;
    if (aProgrammaticName.equalsAscii("cexact"))
	return expression;
    if (aProgrammaticName.equalsAscii("csqrt"))
	return expression;
    if (aProgrammaticName.equalsAscii("csin"))
	return expression;
    if (aProgrammaticName.equalsAscii("ccos"))
	return expression;
    if (aProgrammaticName.equalsAscii("ctan"))
	return expression;
    if (aProgrammaticName.equalsAscii("cln"))
	return expression;
    if (aProgrammaticName.equalsAscii("cexp"))
	return expression;
    if (aProgrammaticName.equalsAscii("cabs"))
	return expression;
    if (aProgrammaticName.equalsAscii("conj"))
	return expression;
    if (aProgrammaticName.equalsAscii("arg"))
	return expression;
    if (aProgrammaticName.equalsAscii("re"))
	return expression;
    if (aProgrammaticName.equalsAscii("im"))
	return expression;
    if (aProgrammaticName.equalsAscii("idivis"))
	return entier;
    if (aProgrammaticName.equalsAscii("evalc"))
	return expression;
    if (aProgrammaticName.equalsAscii("factorial"))
	return entier;
    if (aProgrammaticName.equalsAscii("factoriser"))
	return expression;
    if (aProgrammaticName.equalsAscii("factoriserSurC"))
	return expression;
    if (aProgrammaticName.equalsAscii("developper"))
	return expression;
    if (aProgrammaticName.equalsAscii("divis"))
	return expression;
    if (aProgrammaticName.equalsAscii("factoriserEntier"))
	return entier;
    if (aProgrammaticName.equalsAscii("isprime"))
	return entier;
    if (aProgrammaticName.equalsAscii("carcsin"))
	return expression;
    if (aProgrammaticName.equalsAscii("carccos"))
	return expression;
    if (aProgrammaticName.equalsAscii("carctan"))
	return expression;
    if (aProgrammaticName.equalsAscii("egv"))
	return matrice;
    if (aProgrammaticName.equalsAscii("eigenvals"))
	return matrice;
    if (aProgrammaticName.equalsAscii("matinv"))
	return matrice;
    if (aProgrammaticName.equalsAscii("det"))
	return matrice;

    // pas encore implémenté
    if (aProgrammaticName.equalsAscii("prevprime"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("nextprime"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("isPseudoPrime"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("euler"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("latex"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("mathml"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("developperTranscendant"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("ccosh"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("csinh"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("ctanh"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("ccotan"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("carccosh"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("carcsinh"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("carctanh"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("carccotan"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("iegcd"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("iabcuv"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("egcd"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("abcuv"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("dsolve"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("linsolve"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("cas1"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("cas2"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("cas3"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("cas4"))
	return notYetImplemented;
    if (aProgrammaticName.equalsAscii("cas5"))
	return notYetImplemented;


    // plusieurs paramètres

    if (aProgrammaticName.equalsAscii("fmin"))
   {
   	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("la variable (facultatif)");
			break;
		case 3 : 
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable (optional)"); // to translate in deutsch
                        break;
		default : // english must be default
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable (optional)");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("fmax"))
   {
   	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("la variable (facultatif)");
			break;
		case 3 : // to translate in deutsch
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable (optional)"); // to translate in deutsch
                        break;
		default : // english must be default
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable (optional)");
			break;
	}
	return ArgumentName[nArgument];
   }


   if (aProgrammaticName.equalsAscii("matadd") || aProgrammaticName.equalsAscii("matsub") || aProgrammaticName.equalsAscii("matmult"))
   {
	OUString ArgumentName[2];
	ArgumentName[0]=matrice;
	ArgumentName[1]=matrice;
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("matpow"))
   {
	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 : // french
			ArgumentName[0]=matrice;
			ArgumentName[1]=OUString::createFromAscii("l'exposant");
			break;
		case 3 : // deutsch
			ArgumentName[0]=matrice;
			ArgumentName[1]=OUString::createFromAscii("der Exponent");
			break;
		default : // english must be default
			ArgumentName[0]=matrice;
			ArgumentName[1]=OUString::createFromAscii("the power");
			break;
	}
	return ArgumentName[nArgument];
   }
  



   if (aProgrammaticName.equalsAscii("cpow"))
   {
	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 : // french
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("l'exposant");
			break;
		case 3 : // deutsch
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("der Exponent");
			break;
		default : // english must be default
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("the power");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("subst"))
   {
	OUString ArgumentName[3];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("la variable ")+aGrave+OUString::createFromAscii(" substituer");
			ArgumentName[2]=OUString::createFromAscii("la valeur prise par la variable");
			break;
		case 3 : // deutsch
			 // to translate in deutsch         
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable to substitute");
			ArgumentName[2]=OUString::createFromAscii("variable value");                        
                        break;
		default : // english must be default
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable to substitute");
			ArgumentName[2]=OUString::createFromAscii("variable value");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("evalf"))
   {
   	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("nombre de d")+eAigu+OUString::createFromAscii("cimales");
			break;
		case 3 : // deutsch
			 // to translate in deutsch         
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("number of digits");                        
                        break;
		default : // english must be default
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("number of digits");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("deriver"))
   {
   	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("la variable de d")+eAigu+OUString::createFromAscii("rivation");
			break;
		case 3 : // deutsch
			 // to translate in deutsch         
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable of differentiation");                        
                        break;
		default : // english must be default
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable of differentiation");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("integrer"))
   {
	OUString ArgumentName[4];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("la variable d'int")+eAigu+OUString::createFromAscii("gration");
			ArgumentName[2]=OUString::createFromAscii("la borne a");
			ArgumentName[3]=OUString::createFromAscii("la borne b");
			break;
		case 3 : // deutsch
			 // to translate in deutsch                        
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable of integration");
			ArgumentName[2]=OUString::createFromAscii("from a");
			ArgumentName[3]=OUString::createFromAscii("to b");

                        break;
		default : // english must be default
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable of integration");
			ArgumentName[2]=OUString::createFromAscii("from a");
			ArgumentName[3]=OUString::createFromAscii("to b");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("csomme"))
   {
	OUString ArgumentName[4];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("la variable");
			ArgumentName[2]=OUString::createFromAscii("la borne a");
			ArgumentName[3]=OUString::createFromAscii("la borne b");
			break;
		case 3 : // deutsch
			 // to translate in deutsch                        
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable");
			ArgumentName[2]=OUString::createFromAscii("from a");
			ArgumentName[3]=OUString::createFromAscii("to b");                        
                        break;
		default : // english must be default
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable");
			ArgumentName[2]=OUString::createFromAscii("from a");
			ArgumentName[3]=OUString::createFromAscii("to b");
			break;
	}
	return ArgumentName[nArgument];
   }
   
   if (aProgrammaticName.equalsAscii("cproduit"))
   {
	OUString ArgumentName[4];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("la variable");
			ArgumentName[2]=OUString::createFromAscii("la borne a");
			ArgumentName[3]=OUString::createFromAscii("la borne b");
			break;
		case 3 : // deutsch
			 // to translate in deutsch                        
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable");
			ArgumentName[2]=OUString::createFromAscii("from a");
			ArgumentName[3]=OUString::createFromAscii("to b");                        
                        break;
		default : // english must be default
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable");
			ArgumentName[2]=OUString::createFromAscii("from a");
			ArgumentName[3]=OUString::createFromAscii("to b");
			break;
	}
	return ArgumentName[nArgument];
   }
   
   if (aProgrammaticName.equalsAscii("limite"))
   {
	OUString ArgumentName[4];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("la variable");
			ArgumentName[2]=OUString::createFromAscii("la valeur de a");
			ArgumentName[3]=OUString::createFromAscii("la direction (facultatif)");
			break;
		case 3 : // deutsch
			 // to translate in deutsch                        
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable");
			ArgumentName[2]=OUString::createFromAscii("value");
			ArgumentName[3]=OUString::createFromAscii("direction (optional)");
                        break;
                        
                        break;
		default : // english must be default
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable");
			ArgumentName[2]=OUString::createFromAscii("value");
			ArgumentName[3]=OUString::createFromAscii("direction (optional)");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("makelist"))
   {
	OUString ArgumentName[5];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("la variable");
			ArgumentName[2]=OUString::createFromAscii("la valeur de a");
			ArgumentName[3]=OUString::createFromAscii("la valeur de b");
			ArgumentName[4]=OUString::createFromAscii("le pas (facultatif)");
			break;
		case 3 : // deutsch
			 // to translate in deutsch                        
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable");
			ArgumentName[2]=OUString::createFromAscii("from a");
			ArgumentName[3]=OUString::createFromAscii("to b");
			ArgumentName[4]=OUString::createFromAscii("step (optional)");                      
                        break;
		default : // english must be default
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable");
			ArgumentName[2]=OUString::createFromAscii("from a");
			ArgumentName[3]=OUString::createFromAscii("to b");
			ArgumentName[4]=OUString::createFromAscii("step (optional)");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("czeros"))
   {
   	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("la variable (facultatif)");
			break;
		case 3 : // deutsch
			 // to translate in deutsch
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable (optional)");                       
                        break;
		default : // english must be default
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable (optional)");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("zeros"))
   {
   	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("la variable (facultatif)");
			break;
		case 3 : // deutsch
			 // to translate in deutsch
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable (optional)");                        
                        break;
		default : // english must be default
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable (optional)");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("iquo"))
   {
   	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=OUString::createFromAscii("un entier ou un entier de Gauss");
			ArgumentName[1]=OUString::createFromAscii("un entier ou un entier de Gauss");
			break;
		case 3 : // deutsch
			 // to translate in deutsch
			ArgumentName[0]=OUString::createFromAscii("an integer or a Gauss integer");
			ArgumentName[1]=OUString::createFromAscii("an integer or a Gauss integer");
                        break;
		default : // english must be default
			ArgumentName[0]=OUString::createFromAscii("an integer or a Gauss integer");
			ArgumentName[1]=OUString::createFromAscii("an integer or a Gauss integer");
			break;
	}
	return ArgumentName[nArgument];
   }
   
   if (aProgrammaticName.equalsAscii("irem"))
   {
   	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=OUString::createFromAscii("un entier ou un entier de Gauss");
			ArgumentName[1]=OUString::createFromAscii("un entier ou un entier de Gauss");
			break;
		case 3 : // deutsch
			 // to translate in deutsch                        
			ArgumentName[0]=OUString::createFromAscii("an integer or a Gauss integer");
			ArgumentName[1]=OUString::createFromAscii("an integer or a Gauss integer");
                        break;
		default : // english must be default
			ArgumentName[0]=OUString::createFromAscii("an integer or a Gauss integer");
			ArgumentName[1]=OUString::createFromAscii("an integer or a Gauss integer");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("quo"))
   {
	OUString ArgumentName[3];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=OUString::createFromAscii("un polyn")+oCirconflexe+OUString::createFromAscii("me");
			ArgumentName[1]=OUString::createFromAscii("un polyn")+oCirconflexe+OUString::createFromAscii("me");
			ArgumentName[2]=OUString::createFromAscii("la variable (facultatif)");
			break;
		case 3 : // deutsch
			 // to translate in deutsch                        
			ArgumentName[0]=OUString::createFromAscii("A polynomial");
			ArgumentName[1]=OUString::createFromAscii("a polynomial");
			ArgumentName[2]=OUString::createFromAscii("variable (optional)");
                        break;
		default : // english must be default
			ArgumentName[0]=OUString::createFromAscii("A polynomial");
			ArgumentName[1]=OUString::createFromAscii("a polynomial");
			ArgumentName[2]=OUString::createFromAscii("variable (optional)");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("rem"))
   {
	OUString ArgumentName[3];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=OUString::createFromAscii("un polyn")+oCirconflexe+OUString::createFromAscii("me");
			ArgumentName[1]=OUString::createFromAscii("un polyn")+oCirconflexe+OUString::createFromAscii("me");
			ArgumentName[2]=OUString::createFromAscii("la variable (facultatif)");
			break;
		case 3 : // deutsch
			 // to translate in deutsch
			ArgumentName[0]=OUString::createFromAscii("A polynomial");
			ArgumentName[1]=OUString::createFromAscii("a polynomial");
			ArgumentName[2]=OUString::createFromAscii("variable (optional)");
                        break;
		default : // english must be default
			ArgumentName[0]=OUString::createFromAscii("A polynomial");
			ArgumentName[1]=OUString::createFromAscii("a polynomial");
			ArgumentName[2]=OUString::createFromAscii("variable (optional)");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("resoudre"))
   {
   	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=OUString::createFromAscii("L'")+eAigu+OUString::createFromAscii("quation");
			ArgumentName[1]=OUString::createFromAscii("la variable (facultatif si x)");
			break;
		case 3 : // deutsch
			 // to translate in deutsch                        
			ArgumentName[0]=OUString::createFromAscii("The equation");
			ArgumentName[1]=OUString::createFromAscii("variable (optional if x)");
                        break;
		default : // english must be default
			ArgumentName[0]=OUString::createFromAscii("The equation");
			ArgumentName[1]=OUString::createFromAscii("variable (optional if x)");
			break;
	}
	return ArgumentName[nArgument];
   }
   
   if (aProgrammaticName.equalsAscii("resoudreDansC"))
   {
   	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=OUString::createFromAscii("L'")+eAigu+OUString::createFromAscii("quation");
			ArgumentName[1]=OUString::createFromAscii("la variable (facultatif si x)");
			break;
		case 3 : // deutsch
			 // to translate in deutsch
			ArgumentName[0]=OUString::createFromAscii("The equation");
			ArgumentName[1]=OUString::createFromAscii("variable (optional if x)");                        
                        break;
		default : // english must be default
			ArgumentName[0]=OUString::createFromAscii("The equation");
			ArgumentName[1]=OUString::createFromAscii("variable (optional if x)");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("resoudreNumerique"))
   {
	OUString ArgumentName[4];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=OUString::createFromAscii("L'")+eAigu+OUString::createFromAscii("quation");
			ArgumentName[1]=OUString::createFromAscii("la variable (facultatif)");
			ArgumentName[2]=OUString::createFromAscii("une valeur ou un intervalle de recherche (facultatif)");
			ArgumentName[3]=OUString::createFromAscii("la m")+eAigu+OUString::createFromAscii("thode de recherche (facultatif)");
			break;
		case 3 : // deutsch
			 // to translate in deutsch                        
			ArgumentName[0]=OUString::createFromAscii("The equation");
			ArgumentName[1]=OUString::createFromAscii("variable (optional if x)");
			ArgumentName[2]=OUString::createFromAscii("guess or interval (optional)");
			ArgumentName[3]=OUString::createFromAscii("method (optional)");
                        break;
		default : // english must be default
			ArgumentName[0]=OUString::createFromAscii("The equation");
			ArgumentName[1]=OUString::createFromAscii("variable (optional if x)");
			ArgumentName[2]=OUString::createFromAscii("guess or interval (optional)");
			ArgumentName[3]=OUString::createFromAscii("method (optional)");
			break;
	}
	return ArgumentName[nArgument];
   }
   
   if (aProgrammaticName.equalsAscii("gcd"))
   {
   	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=OUString::createFromAscii("un entier, un rationnel ou un polyn")+oCirconflexe+OUString::createFromAscii("me");
			ArgumentName[1]=OUString::createFromAscii("un entier, un rationnel ou un polyn")+oCirconflexe+OUString::createFromAscii("me");
			break;
		case 3 : // deutsch
			 // to translate in deutsch                        
			ArgumentName[0]=OUString::createFromAscii("an integer, a rational or a polynomial");
			ArgumentName[1]=OUString::createFromAscii("an integer, a rational or a polynomial");
                        break;
		default : // english must be default
			ArgumentName[0]=OUString::createFromAscii("an integer, a rational or a polynomial");
			ArgumentName[1]=OUString::createFromAscii("an integer, a rational or a polynomial");
			break;
	}
	return ArgumentName[nArgument];
   }
   
   if (aProgrammaticName.equalsAscii("lcm"))
   {
   	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=OUString::createFromAscii("un entier, un rationnel ou un polyn")+oCirconflexe+OUString::createFromAscii("me");
			ArgumentName[1]=OUString::createFromAscii("un entier, un rationnel ou un polyn")+oCirconflexe+OUString::createFromAscii("me");
			break;
		case 3 : // deutsch
			 // to translate in deutsch                        
			ArgumentName[0]=OUString::createFromAscii("an integer, a rational or a polynomial");
			ArgumentName[1]=OUString::createFromAscii("an integer, a rational or a polynomial");
                        break;
		default : // english must be default
			ArgumentName[0]=OUString::createFromAscii("an integer, a rational or a polynomial");
			ArgumentName[1]=OUString::createFromAscii("an integer, a rational or a polynomial");
			break;
	}
	return ArgumentName[nArgument];
   }
   
   if (aProgrammaticName.equalsAscii("formeCanonique"))
   {
   	OUString ArgumentName[2];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=OUString::createFromAscii("un polyn")+oCirconflexe+OUString::createFromAscii("me de degr")+eAigu+OUString::createFromAscii(" 2");
			ArgumentName[1]=OUString::createFromAscii("la variable (facultatif si x)");
			break;
		case 3 : // deutsch
			 // to translate in deutsch                        
			ArgumentName[0]=OUString::createFromAscii("A polynomial of degree 2");
			ArgumentName[1]=OUString::createFromAscii("variable (optional if x)");
                        break;
		default : // english must be default
			ArgumentName[0]=OUString::createFromAscii("A polynomial of degree 2");
			ArgumentName[1]=OUString::createFromAscii("variable (optional if x)");
			break;
	}
	return ArgumentName[nArgument];
   }

   if (aProgrammaticName.equalsAscii("taylor"))
   {
	OUString ArgumentName[3];
	switch(lang)
	{
		case 1 :
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable=limite (facultatif, x=0 par d")+eAigu+OUString::createFromAscii("faut)");
			ArgumentName[2]=OUString::createFromAscii("ordre (facultatif, 5 par d")+eAigu+OUString::createFromAscii("faut)");
			break;
		case 3 : // deutsch
			 // to translate in deutsch                        
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable=limit (optional, default : x=0)");
			ArgumentName[2]=OUString::createFromAscii("order (optional, default : 5)");
                        break;
		default : // english must be default
			ArgumentName[0]=expression;
			ArgumentName[1]=OUString::createFromAscii("variable=limit (optional, default : x=0)");
			ArgumentName[2]=OUString::createFromAscii("order (optional, default : 5)");
			break;
	}
	return ArgumentName[nArgument];
   }
   // Rien trouvé ? Cela ne devrait pas se produire
   return OUString::createFromAscii("None");
}
 
OUString SAL_CALL CmathOOoCASAddinImpl::getArgumentDescription(const OUString& aProgrammaticName, sal_Int32 nArgument )
{
	// return getDisplayArgumentName because I've described argument in the name itself for comfort in Calc
	return (CmathOOoCASAddinImpl::getDisplayArgumentName(aProgrammaticName,nArgument));

}
 
OUString SAL_CALL CmathOOoCASAddinImpl::getProgrammaticCategoryName(const OUString& aProgrammaticName )
{
    OUString aRet( RTL_CONSTASCII_USTRINGPARAM("Add-In"));
    return aRet;
}
 
OUString SAL_CALL CmathOOoCASAddinImpl::getDisplayCategoryName(const OUString& aProgrammaticName )
{
    return getProgrammaticCategoryName( aProgrammaticName );
}

